package com.duben.infinitefunjia;

import android.content.Context;
import android.content.Intent;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.multidex.MultiDex;
import androidx.multidex.MultiDexApplication;

import com.baidu.mobads.sdk.api.MobadsPermissionSettings;
import com.downloader.PRDownloader;
import com.downloader.PRDownloaderConfig;
import com.duben.infinitefunjia.ad.TTAdManagerHolder;
import com.duben.infinitefunjia.common.Constant;
import com.duben.infinitefunjia.manager.CsjAppLogManager;
import com.duben.infinitefunjia.manager.ShareSdkManager;
import com.duben.infinitefunjia.manager.TalkingDataManager;
import com.duben.infinitefunjia.manager.UmengManager;
import com.duben.infinitefunjia.net.LoanService;
import com.duben.infinitefunjia.net.V6Service;
import com.duben.infinitefunjia.utils.AppPreferencesManager;
import com.duben.infinitefunjia.utils.LogUtil;
import com.duben.infinitefunjia.utils.MateUtils;
import com.duben.infinitefunjia.utils.ForegroundOrBackground;
import com.orhanobut.logger.AndroidLogAdapter;
import com.orhanobut.logger.FormatStrategy;
import com.orhanobut.logger.Logger;
import com.orhanobut.logger.PrettyFormatStrategy;
import com.tencent.bugly.crashreport.CrashReport;

import java.io.IOException;

import io.reactivex.exceptions.UndeliverableException;
import io.reactivex.functions.Consumer;
import io.reactivex.plugins.RxJavaPlugins;
import rx.Scheduler;
import rx.schedulers.Schedulers;

/**
 * 描述：Application
 * 作者：孟崔广
 * 时间：2020/6/15 17:40
 */
public class MintsApplication extends MultiDexApplication {

    private static Context mContext;
    private static MintsApplication mApplication;
    public static String OAID = "";
    private static LocalBroadcastManager mLocalBroadcatManager;
    private Scheduler defaultSubscribeScheduler;
    private LoanService loanService;
    private V6Service v6Service;

    /**
     * 获取全局context
     */
    public static Context getContext() {
        return mContext;
    }

    public static MintsApplication getApplication() {
        return mApplication;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mApplication = this;
        mContext = this.getApplicationContext();

        // 友盟SDK预初始化函数
        // preInit预初始化函数耗时极少，不会影响App首次冷启动用户体验
        UmengManager.INSTANCE.preInit(this,
                MateUtils.INSTANCE.getAppMetaData(this, "CHANNEL_NAME"));

        // 三方配置
        thirdConfig();

        setRxJavaErrorHandler();
    }

    @Override
    protected void attachBaseContext(Context base) {
        super.attachBaseContext(base);
        MultiDex.install(base);
    }

    /**
     * 三方配置
     */
    public void thirdConfig() {
        // 未同意权限弹窗，不进行SDK初始化
        if (AppPreferencesManager.INSTANCE.get()
                .getBoolean(Constant.LOAN_PERMISSION_FLAG, true)) return;


        // 判断应用是否在前台
        ForegroundOrBackground.init(this);

        // PR下载附件
        initPRDownloader();

        //logger
        BindLogger();

        TTAdManagerHolder.init(this);

        initMiitHelper();

        UmengManager.INSTANCE.initUm(this,
                MateUtils.INSTANCE.getAppMetaData(this, "CHANNEL_NAME"));

        TalkingDataManager.INSTANCE.init(this);

        CsjAppLogManager.INSTANCE.init(this);

        // bugly
        CrashReport.initCrashReport(this, "cbba7599c2", BuildConfig.DEBUG);

        // ShareSdk
        ShareSdkManager.INSTANCE.submitPolicyGrantResult();

        // 百度sdk
        MobadsPermissionSettings.setPermissionReadDeviceID(true);
    }

    /**
     * PR下载附件
     */
    private void initPRDownloader() {
        PRDownloaderConfig config = PRDownloaderConfig.newBuilder()
                .setDatabaseEnabled(true)
                .build();
        PRDownloader.initialize(this, config);
    }

    /**
     * 自定义logger
     */
    private void BindLogger() {
        //自定义logger
        FormatStrategy formatStrategy = PrettyFormatStrategy.newBuilder()
                .showThreadInfo(false)  //是否选择显示线程信息，默认为true
                .methodCount(0)         //方法数显示多少行，默认2行
                .methodOffset(7)        //隐藏方法内部调用到偏移量，默认5
                // .logStrategy(customLog) //打印日志的策略，默认LogCat
                .tag("mints---network")   //自定义TAG全部标签，默认PRETTY_LOGGER
                .build();
        Logger.addLogAdapter(new AndroidLogAdapter(formatStrategy));
    }

    /**
     * app退出
     */
    public void exitApp() {
        Intent intent = new Intent();
        intent.setAction(Constant.ACTION_EXIT_APP);
        intent.addCategory(Intent.CATEGORY_DEFAULT);
        MintsApplication.getLocalBroadcastManager().sendBroadcast(intent);

        System.gc();
    }

    /**
     * 获得LocalBroadcastManager对象
     *
     * @return LocalBroadcastManager对象
     */
    public static LocalBroadcastManager getLocalBroadcastManager() {
        if (mLocalBroadcatManager == null) {
            mLocalBroadcatManager = LocalBroadcastManager.getInstance(mContext);
        }
        return mLocalBroadcatManager;
    }

    public LoanService getLoanService() {
        if (loanService == null) {
            loanService = LoanService.Factory.create();
        }
        return loanService;
    }

    public V6Service getV6Service() {
        if (v6Service == null) {
            v6Service = V6Service.Factory.create();
        }
        return v6Service;
    }

    public Scheduler defaultSubscribeScheduler() {
        if (defaultSubscribeScheduler == null) {
            defaultSubscribeScheduler = Schedulers.io();
        }
        return defaultSubscribeScheduler;
    }

    /**
     * 移动安装联盟 获取OAID等设备标识符
     */
    private void initMiitHelper() {
//        try {
//            MiitHelper miitHelper = new MiitHelper(new MiitHelper.AppIdsUpdater() {
//
//                @Override
//                public void onIdsValid(String ids) {
//                    MintsApplication.OAID = ids;
//
//                    System.out.println("123123123 oaid=" + ids);
//                }
//            });
//            miitHelper.getDeviceIds(MintsApplication.getContext(), true, true, true);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
    }

    private void setRxJavaErrorHandler() {
        if (RxJavaPlugins.getErrorHandler() != null || RxJavaPlugins.isLockdown()) {
            LogUtil.w("caowj", "setRxJavaErrorHandler getErrorHandler()!=null||isLockdown()");
            return;
        }
        RxJavaPlugins.setErrorHandler(new Consumer<Throwable>() {
            @Override
            public void accept(Throwable e) {
                if (e instanceof UndeliverableException) {
                    e = e.getCause();
                    LogUtil.w("caowj", "setRxJavaErrorHandler UndeliverableException=" + e);
                    return;
                } else if ((e instanceof IOException)) {
                    // fine, irrelevant network problem or API that throws on cancellation
                    return;
                } else if (e instanceof InterruptedException) {
                    // fine, some blocking code was interrupted by a dispose call
                    return;
                } else if ((e instanceof NullPointerException) || (e instanceof IllegalArgumentException)) {
                    // that's likely a bug in the application
                    Thread.UncaughtExceptionHandler uncaughtExceptionHandler =
                            Thread.currentThread().getUncaughtExceptionHandler();
                    if (uncaughtExceptionHandler != null) {
                        uncaughtExceptionHandler.uncaughtException(Thread.currentThread(), e);
                    }
                    return;
                } else if (e instanceof IllegalStateException) {
                    // that's a bug in RxJava or in a custom operator
                    Thread.UncaughtExceptionHandler uncaughtExceptionHandler =
                            Thread.currentThread().getUncaughtExceptionHandler();
                    if (uncaughtExceptionHandler != null) {
                        uncaughtExceptionHandler.uncaughtException(Thread.currentThread(), e);
                    }
                    return;
                }
                LogUtil.w("caowj", "setRxJavaErrorHandler unknown exception=" + e);
            }
        });
    }
}
