package com.duben.infinitefunjia.manager;

import static android.content.Context.NOTIFICATION_SERVICE;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Build;
import android.os.IBinder;

import androidx.annotation.RequiresApi;
import androidx.core.app.NotificationCompat;

import com.duben.infinitefunjia.R;
import com.duben.infinitefunjia.common.Constant;
import com.duben.infinitefunjia.ui.activitys.MainActivity;
import com.duben.infinitefunjia.ui.activitys.PixelActivity;

/**
 * 后台进程保活管理类
 */
public class KeepMintsManager {

    public static final String CHINALL_ID = "MINTS_POSITION";

    /**
     * 前台进程的NotificationId
     */
    private final static int GRAY_SERVICE_ID = 1001;

    /**
     * 单例模式
     */
    private static KeepMintsManager instance = new KeepMintsManager();

    /**
     * 1像素的透明Activity
     */
    private PixelActivity activity;

    /**
     * 监听锁屏/解锁的广播（必须动态注册）
     */
    private LockReceiver lockReceiver;

    public static KeepMintsManager getInstance(){
        return instance;
    }

    /**
     * 传入1像素的透明Activity实例
     * @param activity
     */
    public void setKeepMintsActivity(PixelActivity activity){
        this.activity = activity;
    }

    /**
     * 注册锁屏/解锁广播
     * @param context
     */
    public void registerReceiver(Context context){
        lockReceiver = new LockReceiver();
        IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_SCREEN_OFF);
        filter.addAction(Intent.ACTION_USER_PRESENT);
        context.registerReceiver(lockReceiver,filter);
    }

    /**
     * 注销锁屏/解锁广播
     * @param context
     */
    public void unRegisterReceiver(Context context){
        if(lockReceiver!=null){
            context.unregisterReceiver(lockReceiver);
        }
    }

    /**
     * 设置服务为前台服务
     *
     * @param service
     */
    public void setServiceForeground(Service service) {
        //设置service为前台服务，提高优先级
        if (Build.VERSION.SDK_INT < 18) {
            service.startForeground(GRAY_SERVICE_ID, getNotification2(service));
        } else if (Build.VERSION.SDK_INT > 18 && Build.VERSION.SDK_INT < 25) {
            service.startForeground(GRAY_SERVICE_ID, getNotification2(service));
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            service.startForeground(GRAY_SERVICE_ID, getNotification(service));
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    private Notification getNotification(Service service) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel notificationChannel = null;
            notificationChannel = new NotificationChannel(CHINALL_ID,
                    Constant.MINTS_APP_NAME, NotificationManager.IMPORTANCE_HIGH);
            notificationChannel.enableLights(true);
            notificationChannel.setLightColor(Color.RED);
            notificationChannel.setShowBadge(true);
            notificationChannel.setLockscreenVisibility(Notification.VISIBILITY_PUBLIC);
            NotificationManager manager = (NotificationManager) service.getSystemService(NOTIFICATION_SERVICE);
            manager.createNotificationChannel(notificationChannel);
        }
        Intent intent = new Intent(service, MainActivity.class);
        PendingIntent pi;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.S) {
            pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_IMMUTABLE);
        } else {
            pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_ONE_SHOT);
        }
        Notification notification = new Notification.Builder(service).setChannelId(CHINALL_ID)
                .setSmallIcon(R.mipmap.ic_launcher_main)
                .setContentTitle("正在保护你的安全")
                .setContentIntent(pi)
                .setContentText("关闭会导致数据丢失,请谨慎操作")
                .getNotification();
        notification.flags |= Notification.FLAG_NO_CLEAR;
        return notification;
    }

    private Notification getNotification2(Service service) {
        Intent intent = new Intent(service, MainActivity.class);
        PendingIntent pi;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.S) {
            pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_IMMUTABLE);
        } else {
            pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_ONE_SHOT);
        }
        Notification notification = new NotificationCompat.Builder(service, CHINALL_ID)
                .setContentTitle("正在保护你的安全")
                .setContentText("关闭会导致数据丢失,请谨慎操作！")
                .setWhen(System.currentTimeMillis())
                .setContentIntent(pi)
                .setOngoing(true)
                .setLargeIcon(BitmapFactory.decodeResource(service.getResources(), R.mipmap.ic_launcher_main))
                .setAutoCancel(true)
                .build();
        return notification;
    }

    private void startLiveActivity(Context context){
        Intent intent = new Intent(context,PixelActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }

    private void destroyLiveActivity(){
        if(activity!=null){
            activity.finish();
        }
    }

    class LockReceiver extends BroadcastReceiver{
        @Override
        public void onReceive(Context context, Intent intent) {
            switch (intent.getAction()){
                case Intent.ACTION_SCREEN_OFF:
                    startLiveActivity(context);
                    break;
                case Intent.ACTION_USER_PRESENT:
                    destroyLiveActivity();
                    break;
            }
        }
    }

    /**
     * 辅助Service
     */
    public static class GrayInnerService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                startForeground(GRAY_SERVICE_ID, getNotification(this));
            } else {
                startForeground(GRAY_SERVICE_ID, getNotification2(this));
            }
            stopForeground(true);
            stopSelf();
            return super.onStartCommand(intent, flags, startId);
        }

        @Override
        public IBinder onBind(Intent intent) {
            return null;
        }

        @RequiresApi(api = Build.VERSION_CODES.O)
        private Notification getNotification(Service service) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                NotificationChannel notificationChannel = null;
                notificationChannel = new NotificationChannel(CHINALL_ID,
                        Constant.MINTS_APP_NAME, NotificationManager.IMPORTANCE_HIGH);
                notificationChannel.enableLights(true);
                notificationChannel.setLightColor(Color.RED);
                notificationChannel.setShowBadge(true);
                notificationChannel.setLockscreenVisibility(Notification.VISIBILITY_PUBLIC);
                NotificationManager manager = (NotificationManager) service.getSystemService(NOTIFICATION_SERVICE);
                manager.createNotificationChannel(notificationChannel);
            }
            Intent intent = new Intent(service, MainActivity.class);
            PendingIntent pi;
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.S) {
                pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_IMMUTABLE);
            } else {
                pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_ONE_SHOT);
            }
            Notification notification = new Notification.Builder(service).setChannelId(CHINALL_ID)
                    .setSmallIcon(R.mipmap.ic_launcher_main)
                    .setContentTitle("正在保护你的安全")
                    .setContentIntent(pi)
                    .setContentText("关闭会导致数据丢失,请谨慎操作")
                    .getNotification();
            notification.flags |= Notification.FLAG_NO_CLEAR;
            return notification;
        }

        private Notification getNotification2(Service service) {
            Intent intent = new Intent(service, MainActivity.class);
            PendingIntent pi;
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.S) {
                pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_IMMUTABLE);
            } else {
                pi = PendingIntent.getActivity(service, 0, intent, PendingIntent.FLAG_ONE_SHOT);
            }
            Notification notification = new NotificationCompat.Builder(service, CHINALL_ID)
                    .setContentTitle("正在保护你的安全")
                    .setContentText("关闭会导致数据丢失,请谨慎操作！")
                    .setWhen(System.currentTimeMillis())
                    .setContentIntent(pi)
                    .setOngoing(true)
                    .setLargeIcon(BitmapFactory.decodeResource(service.getResources(), R.mipmap.ic_launcher_main))
                    .setAutoCancel(true)
                    .build();
            return notification;
        }
    }
}
