package com.duben.infinitefunjia.mvp.presenters

import com.google.gson.JsonObject
import com.duben.infinitefunjia.common.DeviceInfo
import com.duben.library.net.neterror.Throwable
import com.duben.infinitefunjia.manager.AppHttpManager
import com.duben.infinitefunjia.manager.UserManager
import com.duben.infinitefunjia.mvp.model.*
import com.duben.infinitefunjia.mvp.views.BookCityView
import com.duben.infinitefunjia.utils.DeviceUuidFactory
import com.duben.library.net.neterror.BaseSubscriber
import java.util.HashMap

class BookCityPresenter : BasePresenter<BookCityView>() {

    /**
     * 游客登录
     */
    fun userLogin() {
        val vo = HashMap<String, Any>()
        vo["device"] = DeviceUuidFactory().deviceUuid.toString()
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.visitorlogin(vo),
                object : BaseSubscriber<BaseResponse<UserBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                    }

                    override fun onError(e: Throwable) {
                        if (isLinkView) return
                    }

                    override fun onNext(baseResponse: BaseResponse<UserBean>) {
                        if (isLinkView) return
                        view.hideLoading()
                        val code = baseResponse.getStatus()
                        val message = baseResponse.getMessage()
                        val data: UserBean? = baseResponse.getData()
                        when (code) {
                            200 -> if (data != null) {
                                UserManager.getInstance().saveUserInfo(data)
                                saveTerminalInfo()
                            }
                            else -> {
                                view.showToast(message)
                            }
                        }
                    }
                })
    }

    /**
     * 提交设备信息
     *
     * @param context
     */
    fun saveTerminalInfo() {
        val vo = HashMap<String, Any>()
        val deviceInfo: DeviceInfo = DeviceInfo.instance
        val macAddress: String = deviceInfo.getMacAddress()
        val mac = macAddress.replace(":", "")
        vo["mac"] = mac
        vo["mac1"] = macAddress
        vo["androidid"] = deviceInfo.getAndroidId(null)
        vo["imei"] = deviceInfo.iMEI
        vo["os"] = if (deviceInfo.isHarmonyOS()) "android-HarmonyOS" else "android"
        vo["model"] = deviceInfo.newModel
        vo["uuid"] = DeviceUuidFactory().deviceUuid
        vo["osversion"] = deviceInfo.oSVersion
        vo["appversion"] = deviceInfo.versionName
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.saveTerminalInfo(vo),
                object : BaseSubscriber<BaseResponse<Any>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                    }

                    override fun onError(e: Throwable) {
                        if (isLinkView) return
                    }

                    override fun onNext(baseResponse: BaseResponse<Any>) {
                        if (isLinkView) return
                        val code = baseResponse.status
                        when (code) {
                        }
                    }
                })
    }

    fun getBannerBook() {
        val vo = HashMap<String, Any>()
        vo["flag"] = "carousel"
        vo["quantity"] = "10"
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getRecommendV1(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getBannerBookSuc(baseResponse.data)
                            else -> {
                                view.getBannerBookFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getBannerBookFail()
                    }

                })

    }

    fun getHotBook() {
        val vo = HashMap<String, Any>()
        vo["flag"] = "hot"
        vo["quantity"] = "5"
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getRecommendV1(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getHotBookSuc(baseResponse.data)
                            else -> {
                                view.getHotBookFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getHotBookFail()
                    }

                })

    }

    fun getNewBook() {
        val vo = HashMap<String, Any>()
        vo["flag"] = "newbook"
        vo["quantity"] = "8"
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getRecommendV1(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getNewBookSuc(baseResponse.data)
                            else -> {
                                view.getNewBookFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getNewBookFail()
                    }

                })

    }

    fun getSoltVedio() {
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.firstArticle(),
                object : BaseSubscriber<BaseResponse<FirstBookBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                    }

                    override fun onError(e: Throwable) {
                        if (isLinkView) return
                    }

                    override fun onNext(baseResponse: BaseResponse<FirstBookBean>) {
                        if (isLinkView) return

                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> {
                                view.getSoltVedioSuc(baseResponse.data)
                            }
                        }
                    }
                })
    }

}