package com.duben.infinitefunjia.mvp.presenters

import com.google.gson.JsonObject
import com.duben.library.net.neterror.Throwable
import com.duben.infinitefunjia.manager.AppHttpManager
import com.duben.infinitefunjia.mvp.model.BaseResponse
import com.duben.infinitefunjia.mvp.model.BookContentBean
import com.duben.infinitefunjia.mvp.views.BookReadView
import com.duben.library.net.neterror.BaseSubscriber
import java.util.HashMap

class BookReadPresenter : BasePresenter<BookReadView>() {

    fun getBookDetail(bookId: String) {
        val vo = HashMap<String, Any>()
        vo["id"] = bookId
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getBookDetail(vo),
                object : BaseSubscriber<BaseResponse<BookContentBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookContentBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getBookDetailSuc(baseResponse.data)
                            else -> {
                                view.getBookDetailFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getBookDetailFail()
                    }

                })
    }

    fun addReadRecord(bookId: String) {
        val vo = HashMap<String, Any>()
        vo["articleId"] = bookId
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.addReadRecord(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                        if (isLinkView) return
                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message
                        when (code) {
                            200 -> {}
                            else -> {
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                    }

                })
    }

    fun collect(articleId: String) {
        val vo = HashMap<String, Any>()
        vo["articleId"] = articleId
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.collect(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> {}
                            else -> {
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                    }
                })
    }

    fun cancelCollect(articleId: String) {
        val vo = HashMap<String, Any>()
        vo["articleId"] = articleId
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.collectCancel(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> {}
                            else -> {
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                    }
                })
    }

}