package com.duben.infinitefunjia.mvp.presenters

import com.duben.library.net.neterror.Throwable
import com.duben.infinitefunjia.manager.AppHttpManager
import com.duben.infinitefunjia.mvp.model.BaseResponse
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.views.BookSearchView
import com.duben.library.net.neterror.BaseSubscriber
import java.util.HashMap

class BookSearchPresenter : BasePresenter<BookSearchView>() {

    fun getHotBook() {
        val vo = HashMap<String, Any>()
        vo["flag"] = "hot"
        vo["quantity"] = "8"
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getRecommendV1(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getHotBookSuc(baseResponse.data)
                            else -> {
                                view.getHotBookFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getHotBookFail()
                    }
                })
    }

    fun searchBook(searchName: String, pageNo: Int, pageSize: Int) {
        view.showLoading("加载中...")
        val vo = HashMap<String, Any>()
        vo["searchName"] = searchName
        vo["pageNo"] = pageNo
        vo["pageSize"] = pageSize
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getHomeV1List(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getSearchBookSuc(baseResponse.data)
                            else -> {
                                view.getSearchBookFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getSearchBookFail()
                    }

                })

    }

}