package com.duben.infinitefunjia.mvp.presenters

import com.google.gson.JsonObject
import com.duben.library.net.neterror.Throwable
import com.duben.infinitefunjia.manager.AppHttpManager
import com.duben.infinitefunjia.mvp.model.BaseResponse
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.views.BookShelfView
import com.duben.library.net.neterror.BaseSubscriber
import java.util.HashMap

class BookShelfPresenter : BasePresenter<BookShelfView>() {

    fun getCollectList() {
        val vo = HashMap<String, Any>()
        vo["sss"] = ""
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.collectList(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getCollectListSuc(baseResponse.data)
                            else -> {
                                view.getCollectListFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)

                        view.getCollectListFail()
                    }

                })
    }

    fun cancelCollect(bookId: String) {
        val vo = HashMap<String, Any>()
        vo["articleId"] = bookId
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.collectCancel(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.cancelCollectSuc()
                            else -> {
                                view.cancelCollectFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)

                        view.cancelCollectFail()
                    }

                })
    }

    fun getArticleLike() {
        val vo = HashMap<String, Any>()
        vo["quantity"] = "8"
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getArticleLike(vo),
                object : BaseSubscriber<BaseResponse<BookListBean>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<BookListBean>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getArticleLikeSuc(baseResponse.data)
                            else -> {
                                view.getArticleLikeFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getArticleLikeFail()
                    }

                })

    }

}