package com.duben.infinitefunjia.mvp.presenters

import com.google.gson.JsonObject
import com.duben.library.net.neterror.BaseSubscriber
import com.duben.library.net.neterror.Throwable
import com.duben.infinitefunjia.manager.AppHttpManager
import com.duben.infinitefunjia.mvp.model.BannerList
import com.duben.infinitefunjia.mvp.model.BaseResponse
import com.duben.infinitefunjia.mvp.model.BookList2Bean
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.views.WatchRecordView
import java.util.HashMap

class WatchRecordPresenter : BasePresenter<WatchRecordView>() {

    fun getHistory() {
        val vo = hashMapOf<String, Any>()
        vo["page"] = 1
        vo["size"] = 100
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.history(vo), object : BaseSubscriber<BaseResponse<BookList2Bean>>() {
                override fun onCompleted() {
                    if (isLinkView) return
                    view.hideLoading()
                }

                override fun onNext(baseResponse: BaseResponse<BookList2Bean>) {
                    if (isLinkView) return

                    view.hideLoading()
                    val code = baseResponse.status
                    val message = baseResponse.message

                    when (code) {
                        200 -> view.getHistorySuc(baseResponse.data)
                        else -> {
                            view.getHistoryFail()
                            view.showToast(message)
                        }
                    }
                }

                override fun onError(e: Throwable?) {
                    if (isLinkView) return
                    view.hideLoading()
                    view.showToast(e?.message)

                    view.getHistoryFail()
                }

            })

    }

    fun collect(articleId: String) {
        val vo = HashMap<String, Any>()
        vo["articleId"] = articleId
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.collect(vo), object : BaseSubscriber<BaseResponse<JsonObject>>() {
                override fun onCompleted() {
                    if (isLinkView) return
                    view.hideLoading()
                }

                override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                    if (isLinkView) return

                    view.hideLoading()
                    val code = baseResponse.status
                    val message = baseResponse.message

                    when (code) {
                        200 -> view.collectSuc()
                        else -> {
                            view.collectFail()
                            view.showToast(message)
                        }
                    }
                }

                override fun onError(e: Throwable?) {
                    if (isLinkView) return
                    view.hideLoading()
                    view.showToast(e?.message)

                    view.collectFail()
                }

            })

    }

    fun cancelCollect(articleId: String) {
        val vo = HashMap<String, Any>()
        vo["articleId"] = articleId
        AppHttpManager.getInstance(loanApplication)
            .call(
                loanService.collectCancel(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.cancelCollectSuc()
                            else -> {
                                view.cancelCollectFail()
                                view.showToast(message)
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)

                        view.cancelCollectFail()
                    }

                })

    }


}