package com.duben.infinitefunjia.net;

import android.text.TextUtils;

import com.duben.infinitefunjia.mvp.model.BookBean;
import com.duben.infinitefunjia.mvp.model.FirstBookBean;
import com.google.gson.JsonObject;
import com.duben.infinitefunjia.BuildConfig;
import com.duben.infinitefunjia.mvp.model.BannerList;
import com.duben.infinitefunjia.mvp.model.BaseResponse;
import com.duben.infinitefunjia.mvp.model.BookChapterBean;
import com.duben.infinitefunjia.mvp.model.BookList2Bean;
import com.duben.infinitefunjia.mvp.model.HotBookTypeList;
import com.duben.infinitefunjia.mvp.model.BookContentBean;
import com.duben.infinitefunjia.mvp.model.BookListBean;
import com.duben.infinitefunjia.mvp.model.OrderRecordBean;
import com.duben.infinitefunjia.mvp.model.UserBean;
import com.duben.infinitefunjia.mvp.model.Version;
import com.duben.infinitefunjia.mvp.model.VipBean;
import com.duben.infinitefunjia.mvp.model.WxPayParamBean;
import com.duben.infinitefunjia.utils.AESUtils;
import com.orhanobut.logger.Logger;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import io.reactivex.Single;
import okhttp3.OkHttpClient;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava.RxJavaCallAdapterFactory;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.http.Body;
import retrofit2.http.POST;
import rx.Observable;

/**
 * 描述：服务器交互
 * 作者：孟崔广
 */
public interface LoanService {

    /**
     * 更新版本
     *
     * @return
     */
    @POST("appNa/checkUpgrade")
    Observable<BaseResponse<Version>> checkUpgrade(@Body Map<String, Object> vo);

    /**
     * 游客登录
     *
     * @return
     */
    @POST("api/user/visitorlogin")
    Observable<BaseResponse<UserBean>> visitorlogin(@Body Map<String, Object> vo);

    /**
     * 获取验证码
     *
     * @return
     */
    @POST("api/user/sendMobileCode")
    Observable<BaseResponse<Object>> sendMobileCode(@Body Map<String, Object> vo);

    /**
     * 登录
     *
     * @return
     */
    @POST("api/user/mobilelogin")
    Observable<BaseResponse<UserBean>> login(@Body Map<String, Object> vo);

    /**
     * 微信登录
     *
     * @return
     */
    @POST("api/user/wechatlogin")
    Observable<BaseResponse<UserBean>> wechatlogin(@Body Map<String, Object> vo);

    /**
     * 提交设备信息
     *
     * @return
     */
    @POST("api/user/saveTerminalInfo")
    Observable<BaseResponse<Object>> saveTerminalInfo(@Body Map<String, Object> vo);

    /**
     * 查询vip产品
     */
    @POST("api/vip/getVipProducts/appv2")
    Observable<BaseResponse<VipBean>> getVipProducts(@Body Map<String, Object> vo);

    /**
     * 获取支付参数
     *
     * @return
     */
    @POST("api/vip/getVipPayParams")
    Observable<BaseResponse<WxPayParamBean>> getVipPayParams(@Body Map<String, Object> vo);

    /**
     * 查询vip订单
     *
     * @return
     */
    @POST("api/vip/queryVipOrder")
    Observable<BaseResponse<Object>> queryVipOrder(@Body Map<String, Object> vo);

    /**
     * 订单记录
     *
     * @return
     */
    @POST("api/vip/getOrderList")
    Observable<BaseResponse<OrderRecordBean>> getCashOutRecords();

    /**
     * 我的
     *
     * @return
     */
    @POST("api/user/baseMsg")
    Observable<BaseResponse<UserBean>> getMyInfo(@Body Map<String, Object> vo);

    /**
     * 取消自动续费
     *
     * @return
     */
    @POST("api/vip/unSign")
    Observable<BaseResponse<JsonObject>> unSign();

    /**
     * 查询失败
     *
     * @return
     */
    @POST("api/vip/payError")
    Observable<BaseResponse<Object>> payError(@Body Map<String, Object> vo);

    /**
     * 排行榜
     *
     * @return
     */
    @POST("api/vedio/orders")
    Observable<BaseResponse<BannerList>> orders();

    /**
     * 阅读记录
     *
     * @return
     */
    @POST("api/book/readRecordList")
    Observable<BaseResponse<BookList2Bean>> history(@Body Map<String, Object> vo);


    /**
     * 收藏列表
     *
     * @return
     */
    @POST("api/book/collectList")
    Observable<BaseResponse<BookListBean>> collectList(@Body Map<String, Object> vo);

    /**
     * 收藏
     *
     * @return
     */
    @POST("api/book/collect")
    Observable<BaseResponse<JsonObject>> collect(@Body Map<String, Object> vo);

    /**
     * 取消收藏
     *
     * @return
     */
    @POST("api/book/collectCancel")
    Observable<BaseResponse<Object>> collectCancel(@Body Map<String, Object> vo);

    /**
     * 书库列表 和 搜索
     *
     * @return
     */
    @POST("api/book/articleList")
    Observable<BaseResponse<BookListBean>> getHomeV1List(@Body Map<String, Object> vo);

    /**
     * 书库分类
     *
     * @return
     */
    @POST("api/book/categoryList")
    Observable<BaseResponse<HotBookTypeList>> getHomeV1Types();

    /**
     * 获取推荐书籍
     *
     * @return
     */
    @POST("api/book/recommendv1")
    Observable<BaseResponse<BookListBean>> getRecommendV1(@Body Map<String, Object> vo);

    /**
     * 获取文章详情
     *
     * @return
     */
    @POST("api/book/info")
    Observable<BaseResponse<BookContentBean>> getBookDetail(@Body Map<String, Object> vo);

    /**
     * 获取章节数据
     *
     * @return
     */
    @POST("api/book/chapterinfo")
    Single<BaseResponse<BookChapterBean>> getChapterInfo(@Body Map<String, Object> vo);

    /**
     * 添加阅读记录
     *
     * @return
     */
    @POST("api/book/addReadRecord")
    Observable<BaseResponse<JsonObject>> addReadRecord(@Body Map<String, Object> vo);

    /**
     * 猜你喜欢
     *
     * @return
     */
    @POST("api/book/articleLike")
    Observable<BaseResponse<BookListBean>> getArticleLike(@Body Map<String, Object> vo);

    /**
     * 搜索书籍
     *
     * @return
     */
    @POST("api/book/articleList")
    Observable<BaseResponse<BookListBean>> getArticleList(@Body Map<String, Object> vo);

    /**
     * 书库分类
     *
     * @return
     */
    @POST("api/book/firstArticle")
    Observable<BaseResponse<FirstBookBean>> firstArticle();

    /**
     * 默认http工厂
     */
    class Factory {
        public static LoanService create() {
            OkHttpClient.Builder builder = new OkHttpClient.Builder();
            builder.retryOnConnectionFailure(true);
            builder.connectTimeout(20, TimeUnit.SECONDS);
            builder.readTimeout(20, TimeUnit.SECONDS);
            builder.writeTimeout(20, TimeUnit.SECONDS);

//            if (BuildConfig.DEBUG) {
//                HttpLoggingInterceptor logging = new HttpLoggingInterceptor();
//                logging.setLevel(HttpLoggingInterceptor.Level.BODY);
//                builder.interceptors().add(logging);
//            }

            if (BuildConfig.DEBUG) {
                HttpLoggingInterceptor logging = new HttpLoggingInterceptor(message -> {
                    if (TextUtils.isEmpty(message)) return;
                    String s = message.substring(0, 1);
                    String request = message.substring(0, 4);
                    if ("{".equals(s) || "[".equals(s)) {
                        Logger.json(message);
                    } else if (request.contains("-->") || request.contains("<--")) {
                        Logger.d("Method" + message);
                    } else {
                        Logger.d("params:" + message);
                    }
                });
                logging.setLevel(HttpLoggingInterceptor.Level.BODY);
                builder.interceptors().add(logging);
            }

            OkHttpInterceptor okHttpInterceptor = new OkHttpInterceptor(AESUtils.getDefaultKey());
            builder.interceptors().add(okHttpInterceptor);

            Retrofit retrofit = new Retrofit.Builder()
                    .client(builder.build())
                    .baseUrl(BuildConfig.MainIp)
                    .addConverterFactory(GsonConverterFactory.create())
                    .addCallAdapterFactory(RxJavaCallAdapterFactory.create())
                    .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                    .build();


            return retrofit.create(LoanService.class);
        }
    }
}
