package com.duben.infinitefunjia.read;

import android.text.TextUtils;
import android.util.Log;

import com.duben.infinitefunjia.MintsApplication;
import com.duben.infinitefunjia.manager.AppHttpManager;
import com.duben.infinitefunjia.mvp.model.BaseResponse;
import com.duben.infinitefunjia.mvp.model.BookChapterBean;
import com.duben.infinitefunjia.mvp.model.BookContentBean;
import com.duben.infinitefunjia.read.models.BookModel;
import com.duben.infinitefunjia.read.models.TxtChapterModel;
import com.duben.library.net.neterror.BaseSubscriber;
import com.duben.library.net.neterror.Throwable;

import org.reactivestreams.Subscriber;
import org.reactivestreams.Subscription;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.reactivex.Flowable;
import io.reactivex.Single;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;


public class NetworkPageLoader extends PageLoader {
    private Subscription mChapterSub;
    private boolean isSkip = false;

    public NetworkPageLoader(YPageView pageView, BookModel bookModel) {
        super(pageView, bookModel);
    }

    @Override
    public void initData() {
        loadCurrentChapter(curChapterIndex);
    }

    @Override
    public BufferedReader getChapterReader(TxtChapterModel chapterModel) throws IOException {
        File file = new File(Constants.BOOK_CACHE_PATH + bookModel.getBookId()
                + File.separator + chapterModel.title + FileUtils.SUFFIX_NB);
        if (!file.exists()) return null;
        Reader reader = new FileReader(file);
        return new BufferedReader(reader);
    }

    //加载章节列表
    @Override
    public void loadChapters() {
        //已缓存
        if (mChapterList != null && !mChapterList.isEmpty()) {
            loadCurrentChapter(curChapterIndex);
            return;
        }

        Map<String, Object> params = new HashMap<>();
        params.put("id", bookModel.getBookId());

        MintsApplication application = MintsApplication.getApplication();

        AppHttpManager.getInstance(application).call(
                application.getLoanService().getBookDetail(params),
                new BaseSubscriber<BaseResponse<BookContentBean>>() {
                    @Override
                    public void onNext(BaseResponse<BookContentBean> baseResponse) {
                        List<BookContentBean.ArticleChapterList> articleChapterList =
                                baseResponse.getData().getArticleChapterList();
                        BookContentBean contentBean = baseResponse.getData();
                        bookModel.setBookTitle(contentBean.getTitle());

                        for (BookContentBean.ArticleChapterList chapterList : articleChapterList) {
                            TxtChapterModel txtChapterModel = new TxtChapterModel();
                            txtChapterModel.setChapterId("" + chapterList.getId());
                            txtChapterModel.setBook_id(bookModel.getId());
                            txtChapterModel.setLock(chapterList.getIsVip() == 1);
                            if (TextUtils.isEmpty(chapterList.getChapterName())) {
                                txtChapterModel.setTitle("第" + chapterList.getChapterNum() + "章");
                            } else {
                                txtChapterModel.setTitle(chapterList.getChapterName());
                            }
                            mChapterList.add(txtChapterModel);
                        }

                        chapterChangeListener.finishedLoadChapters(mChapterList);
                    }

                    @Override
                    public void onCompleted() {

                    }

                    @Override
                    public void onError(Throwable e) {

                    }
                }
        );
    }

    @Override
    public void loadNextChapters() {
        if (chapterChangeListener != null) {
            // 提示加载后两章
            int begin = curChapterIndex + 1;
            int end = begin + 1;

            // 判断是否大于最后一章
            if (begin >= mChapterList.size()) {
                // 如果下一章超出目录了，就没有必要加载了
                return;
            }
            if (end > mChapterList.size()) {
                end = mChapterList.size() - 1;
            }
            requestChapters(begin, end);
        }
    }

    @Override
    public void loadPreChapters() {
        // 加载当前页的前面两个章节
        if (chapterChangeListener != null) {
            int end = curChapterIndex;
            int begin = end - 2;
            if (begin < 0) {
                begin = 0;
            }

            requestChapters(begin, end);
        }
    }

    @Override
    public void preChapter() {
        super.preChapter();
        loadPreChapters();
    }

    @Override
    public void nextChapter() {
        super.nextChapter();
        loadNextChapters();
    }

    @Override
    public void skipToChapter(int pos) {
        if (Math.abs(pos - curChapterIndex) > 1) {
            curChapterIndex = pos;
            isSkip = true;
            loadCurrentChapter(pos);
        } else {
            skipToChapterReal(pos);
        }
    }

    private void skipToChapterReal(int pos) {
        super.skipToChapter(pos);
        loadCurrentChapter(pos);
    }

    /**
     * 加载前一页，当前页，后一页。
     */
    public void loadCurrentChapter(int cur) {
        if (chapterChangeListener != null) {
            int begin = cur;
            int end = cur;

            // 是否当前不是最后一章
            if (end < mChapterList.size()) {
                end = end + 1;
                if (end >= mChapterList.size()) {
                    end = mChapterList.size() - 1;
                }
            }

            // 如果当前不是第一章
            if (begin != 0) {
                begin = begin - 1;
                if (begin < 0) {
                    begin = 0;
                }
            }

            requestChapters(begin, end);
        }
    }

    private void requestChapters(int start, int end) {
        // 检验输入值
        if (start < 0) {
            start = 0;
        }

        if (end >= mChapterList.size()) {
            end = mChapterList.size() - 1;
        }

        List<TxtChapterModel> chapters = new ArrayList<>();

        // 过滤，哪些数据已经加载了
        for (int i = start; i <= end; ++i) {
            TxtChapterModel txtChapter = mChapterList.get(i);
            if (!hasChapterData(txtChapter)) {
                chapters.add(txtChapter);
            }
        }

        if (!chapters.isEmpty()) {
            chapterChangeListener.requestChapters(chapters);
            loadChapter(String.valueOf(bookModel.getBookId()), chapters);
        } else {
            if (isSkip) isSkip = false;
            pageView.showContent();
        }
    }

    private boolean hasChapterData(TxtChapterModel chapter) {
        return BookManager.isChapterCached(String.valueOf(bookModel.getBookId()), chapter.title);
    }

    //加载章节具体内容
    private void loadChapter(String bookId, List<TxtChapterModel> bookChapters) {
        int size = bookChapters.size();

        //取消上次的任务，防止多次加载
        if (mChapterSub != null) {
            mChapterSub.cancel();
        }

        List<Single<BaseResponse<BookChapterBean>>> chapterInfos = new ArrayList<>(size);
        ArrayDeque<String> titles = new ArrayDeque<>(size);

        for (int i = 0; i < size; ++i) {
            TxtChapterModel bookChapter = bookChapters.get(i);
            if (!bookChapter.isLock) {
                Log.v("yzll", "缓存章节：" + bookChapter.getTitle());
                Map<String, Object> params = new HashMap<>();
                params.put("articleId", bookModel.getBookId());
                params.put("chapterId", bookChapter.getChapterId());
                Single<BaseResponse<BookChapterBean>> chapterInfoSingle = MintsApplication.getApplication().getLoanService().getChapterInfo(params).map(bean -> bean);
                chapterInfos.add(chapterInfoSingle);
                titles.add(bookChapter.getTitle());
            }
        }

        Flowable<BaseResponse<BookChapterBean>> concat = Single.concat(chapterInfos);
        concat.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                        new Subscriber<BaseResponse<BookChapterBean>>() {
                            String title = titles.poll();

                            @Override
                            public void onSubscribe(Subscription s) {
                                s.request(Integer.MAX_VALUE);
                                mChapterSub = s;
                            }

                            @Override
                            public void onNext(BaseResponse<BookChapterBean> bookChapterBean) {
                                //存储数据
                                saveChapterInfo(bookId, title, bookChapterBean.getData().getContent());
                                //将获取到的数据进行存储
                                title = titles.poll();
                            }

                            @Override
                            public void onError(java.lang.Throwable t) {
                            }

                            @Override
                            public void onComplete() {
                                if (isSkip) {
                                    isSkip = false;
                                    skipToChapterReal(curChapterIndex);
                                } else {
                                    pageView.showContent();
                                }
                            }
                        });
    }

    /**
     * 存储章节
     *
     * @param folderName
     * @param fileName
     * @param content
     */
    public void saveChapterInfo(String folderName, String fileName, String content) {
        File file = BookManager.getBookFile(folderName, fileName);
        //获取流并存储
        Writer writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(file));
            writer.write(content);
            writer.flush();
        } catch (IOException e) {
            e.printStackTrace();
            close(writer);
        }
    }

    //关闭数据流
    private void close(Closeable closeable) {
        if (closeable == null) return;
        try {
            closeable.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
