package com.duben.infinitefunjia.ui.activitys

import android.os.Bundle
import android.view.View
import androidx.core.content.ContextCompat
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.BOOK_CONTENT_COLOR
import com.duben.infinitefunjia.common.BOOK_CONTENT_SIZE
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.ReadTimeCount
import com.duben.infinitefunjia.mvp.model.BookContentBean
import com.duben.infinitefunjia.mvp.presenters.BookReadPresenter
import com.duben.infinitefunjia.mvp.views.BookReadView
import com.duben.infinitefunjia.ui.activitys.base.BaseActivity
import com.duben.infinitefunjia.ui.widgets.BookSettingDialog
import com.duben.infinitefunjia.utils.AppPreferencesManager
import com.duben.library.utils.GlideUtils
import kotlinx.android.synthetic.main.activity_book_read.*
import kotlinx.android.synthetic.main.header_layout.*


class BookReadActivity : BaseActivity(), View.OnClickListener, BookReadView {

    private var bookId = ""
    private var isCollect = false
    private val readPresenter by lazy { BookReadPresenter() }

    private var bookSettingDialog: BookSettingDialog? = null

    override fun getContentViewLayoutID() = R.layout.activity_book_read

    override fun isApplyKitKatTranslucency() = false

    override fun getBundleExtras(extras: Bundle?) {
        super.getBundleExtras(extras)
        extras?.let {
            bookId = extras.getString(Constant.ROUTE_BOOK_ID).toString()
        }
    }

    override fun initViewsAndEvents() {
        iv_left_icon.visibility = View.VISIBLE
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_back)
        readPresenter.attachView(this)

        initListener()
        popBookSettingDialog()
    }

    override fun onResume() {
        super.onResume()
        readPresenter.getBookDetail(bookId)
        readPresenter.addReadRecord(bookId)
        ReadTimeCount.instance.startCountReadTime()
    }

    override fun onPause() {
        super.onPause()
        ReadTimeCount.instance.endCountReadTime()
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_left_icon -> finish()
            R.id.tab_setting_collect -> {
                setIsCollect(!isCollect)
            }
            R.id.tab_setting_book -> {
                bookSettingDialog?.show()
            }
            R.id.tab_setting_share -> {

            }
            R.id.btn_submit -> {
                val bundle = Bundle()
                bundle.putBoolean(VipActivity.IS_RECOMMEND, true)
                readyGo(VipActivity::class.java, bundle)
            }
        }
    }

    private fun popBookSettingDialog() {
        bookSettingDialog = BookSettingDialog(this)
        bookSettingDialog?.setOnBookSettingCallback(object :
            BookSettingDialog.OnBookSettingCallback {
            override fun onFontSize(size: Int) {
                tv_content.textSize = size.toFloat()
                AppPreferencesManager.get().put(BOOK_CONTENT_SIZE, size)
            }

            override fun onBgColor(color: Int) {
                view_book.setBackgroundColor(color)
                AppPreferencesManager.get().put(BOOK_CONTENT_COLOR, color)
            }
        })
        bookSettingDialog?.loadSetting()
    }

    private fun initListener() {
        iv_left_icon.setOnClickListener(this)
        tab_setting_collect.setOnClickListener(this)
        tab_setting_share.setOnClickListener(this)
        tab_setting_book.setOnClickListener(this)
        btn_submit.setOnClickListener(this)
    }

    override fun getBookDetailSuc(data: BookContentBean) {
        setBookThumb(data)

        if (data.isUnlock == 0) {
            view_vip.visibility = View.VISIBLE
            tv_content.text = data.freeMd
        } else {
            view_vip.visibility = View.GONE
            tv_content.text = data.freeMd + data.chargeMd
        }
    }

    private fun setBookThumb(data: BookContentBean) {
        tv_title.text = "阅读:${data.title}"

        GlideUtils.loadImageViewNoAnim2(this, data.avatar, image_iv)
        title_tv.text = data.title
        info_tv.text = data.summary
        label_tv.text = data.author

        if (data.isCollect == 1) {
            tab_iv_bookshelf.setImageResource(R.mipmap.ic_book_collect_s)
            tab_tv_bookshelf.text = "已收藏"
            tab_tv_bookshelf.setTextColor(ContextCompat.getColor(this, R.color.color_goldenrod))
        } else {
            tab_iv_bookshelf.setImageResource(R.mipmap.ic_book_collect)
            tab_tv_bookshelf.text = "收藏"
            tab_tv_bookshelf.setTextColor(ContextCompat.getColor(this, R.color.black))
        }
    }

    private fun setIsCollect(collect: Boolean) {
        isCollect = collect
        if (isCollect) {
            tab_iv_bookshelf.setImageResource(R.mipmap.ic_book_collect_s)
            tab_tv_bookshelf.text = "已收藏"
            tab_tv_bookshelf.setTextColor(ContextCompat.getColor(this, R.color.color_goldenrod))
            readPresenter.collect(bookId)
        } else {
            tab_iv_bookshelf.setImageResource(R.mipmap.ic_book_collect)
            tab_tv_bookshelf.text = "收藏"
            tab_tv_bookshelf.setTextColor(ContextCompat.getColor(this, R.color.black))
            readPresenter.cancelCollect(bookId)
        }
    }

    override fun getBookDetailFail() {

    }

    override fun onDestroy() {
        super.onDestroy()
        readPresenter.detachView()
    }
}
