package com.duben.infinitefunjia.ui.activitys

import android.text.Editable
import android.text.TextWatcher
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.flexbox.FlexWrap
import com.google.android.flexbox.FlexboxLayoutManager
import com.google.android.flexbox.JustifyContent
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.BookReadManager
import com.duben.infinitefunjia.mvp.model.BookBean
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.presenters.BookSearchPresenter
import com.duben.infinitefunjia.mvp.views.BookSearchView
import com.duben.infinitefunjia.ui.activitys.base.BaseActivity
import com.duben.infinitefunjia.ui.adapter.SearchBookAdapter
import com.duben.infinitefunjia.ui.adapter.SearchHistoryAdapter
import com.duben.infinitefunjia.ui.adapter.SearchHotBookAdapter
import com.duben.infinitefunjia.utils.AppPreferencesManager
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import kotlinx.android.synthetic.main.activity_book_search.*
import kotlinx.android.synthetic.main.header_layout.*

class BookSearchActivity : BaseActivity(), View.OnClickListener, BookSearchView, OnRefreshListener,
    OnLoadMoreListener {

    companion object {
        const val BOOK_SEARCH_HISTORY = "SEARCH_HISTORY"
    }

    private val searchPresenter by lazy { BookSearchPresenter() }

    private var searchBookAdapter: SearchBookAdapter? = null
    private var searchBookData = arrayListOf<BookBean>()
    private var searchPage = 1 // 分页
    private var searchPageSize = Constant.PAGE_SIZE // 分页

    private var searchHotBookAdapter: SearchHotBookAdapter? = null
    private var searchHotBookData = arrayListOf<BookBean>()

    private var searchHistoryAdapter: SearchHistoryAdapter? = null
    private var searchHistoryData = arrayListOf<String>()

    override fun getContentViewLayoutID() = R.layout.activity_book_search

    override fun isApplyKitKatTranslucency() = false

    override fun initViewsAndEvents() {
        tv_title.text = "书库搜索"
        iv_left_icon.visibility = View.VISIBLE
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_back)
        searchPresenter.attachView(this)

        initListener()
        initSearchBookAdapter()
        initSearchHotBookAdapter()
        initSearchHistoryAdapter()

        searchPresenter.getHotBook()
        getSearchHistory()
    }

    private fun initSearchHistoryAdapter() {
        searchHistoryAdapter = SearchHistoryAdapter(this)
        searchHistoryAdapter?.setOnItemClickListener { _, _, position ->
            et_search.setText(searchHistoryData[position])
        }
        val layoutManager = FlexboxLayoutManager(this)
        layoutManager.flexWrap = FlexWrap.WRAP
        layoutManager.justifyContent = JustifyContent.FLEX_START
        rv_search_history.layoutManager = layoutManager
        rv_search_history.adapter = searchHistoryAdapter
    }

    private fun initSearchHotBookAdapter() {
        searchHotBookAdapter = SearchHotBookAdapter(this)
        searchHotBookAdapter?.setOnItemClickListener { _, _, position ->
            BookReadManager.instance.goReadPage(this, searchBookData[position])
        }
        rv_hot.layoutManager = LinearLayoutManager(this)
        rv_hot.adapter = searchHotBookAdapter
    }

    private fun initSearchBookAdapter() {
        searchBookAdapter = SearchBookAdapter(this)
        searchBookAdapter?.setOnItemClickListener { _, _, position ->
            BookReadManager.instance.goReadPage(this, searchBookData[position])
        }
        rv_search_show.layoutManager = LinearLayoutManager(this)
        rv_search_show.adapter = searchBookAdapter
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_left_icon -> finish()
            R.id.btn_search -> {
                searchBook(et_search.text.toString())
            }
            R.id.iv_clear -> {
                clearSearchHistory()
            }
            R.id.iv_change -> {
                searchPresenter.getHotBook()
            }
        }
    }

    private fun initListener() {
        srl_show.setOnRefreshListener(this)
        srl_show.setOnLoadMoreListener(this)
        iv_left_icon.setOnClickListener(this)
        btn_search.setOnClickListener(this)
        iv_clear.setOnClickListener(this)
        iv_change.setOnClickListener(this)

        et_search.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                if (s != null && s.toString() != "") {
                    addSearchHistory(s.toString())
                    searchBook(s.toString())
                    fl_main.visibility = View.GONE
                    fl_search_show.visibility = View.VISIBLE
                } else {
                    fl_main.visibility = View.VISIBLE
                    fl_search_show.visibility = View.GONE
                }
            }

            override fun afterTextChanged(s: Editable?) {
            }
        })
    }

    private fun searchBook(book: String) {
        searchPresenter.searchBook(book, searchPage, searchPageSize)
    }

    override fun getHotBookSuc(data: BookListBean) {
        searchHotBookData.clear()
        searchHotBookData.addAll(data.records)
        searchHotBookAdapter?.setList(searchHotBookData)
    }

    override fun getHotBookFail() {

    }

    override fun getSearchBookSuc(data: BookListBean) {
        if (searchPage == 1) {
            if (data.records.size > 0) {
                search_show_empty.visibility = View.GONE
                srl_show.visibility = View.VISIBLE
            } else {
                search_show_empty.visibility = View.VISIBLE
                srl_show.visibility = View.GONE
            }

            srl_show.finishRefresh(true)
            searchBookData.clear()
            searchBookData.addAll(data.records)
            searchBookAdapter?.setList(data.records)
        } else {
            if (data.records.size < searchPageSize) {
                srl_show.finishLoadMoreWithNoMoreData()
            } else {
                srl_show.finishLoadMore()
            }
            searchBookData.addAll(data.records)
            searchBookAdapter?.addData(data.records)
        }
    }

    override fun getSearchBookFail() {
        search_show_empty.visibility = View.GONE
        rv_search_show.visibility = View.VISIBLE
    }

    private fun addSearchHistory(str: String) {
        var searchHistory = AppPreferencesManager.get().getString(BOOK_SEARCH_HISTORY, "")
        searchHistory += ",$str"
        AppPreferencesManager.get().put(BOOK_SEARCH_HISTORY, searchHistory)

        rv_search_history.visibility = View.VISIBLE
        search_empty.visibility = View.GONE

        val searchList = searchHistory!!.split(",")
        searchHistoryData.clear()
        for (s in searchList) {
            if (s != "") {
                searchHistoryData.add(s)
            }
        }
        searchHistoryAdapter?.notifyDataSetChanged()
    }

    private fun getSearchHistory() {
        val searchHistory = AppPreferencesManager.get().getString(BOOK_SEARCH_HISTORY, "")
        if (searchHistory == "") {
            rv_search_history.visibility = View.GONE
            search_empty.visibility = View.VISIBLE
        } else {
            rv_search_history.visibility = View.VISIBLE
            search_empty.visibility = View.GONE
            val searchList = searchHistory!!.split(",")
            searchHistoryData.clear()
            for (s in searchList) {
                if (s != "") {
                    searchHistoryData.add(s)
                }
            }
            searchHistoryAdapter?.setNewInstance(searchHistoryData)
        }
    }

    private fun clearSearchHistory() {
        AppPreferencesManager.get().put(BOOK_SEARCH_HISTORY, "")
        searchHistoryData.clear()
        searchHistoryAdapter?.notifyDataSetChanged()
        rv_search_history.visibility = View.GONE
        search_empty.visibility = View.VISIBLE
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        searchPage = 1
        srl_show.resetNoMoreData()
        searchBook(et_search.text.toString())
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        ++searchPage
        searchBook(et_search.text.toString())
    }

    override fun onDestroy() {
        super.onDestroy()
        searchPresenter.detachView()
    }
}
