package com.duben.infinitefunjia.ui.activitys

import android.app.Dialog
import android.graphics.Color
import android.os.Bundle
import android.view.View
import androidx.core.view.GravityCompat
import androidx.recyclerview.widget.LinearLayoutManager
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.BookConstant
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.ReadTimeCount
import com.duben.infinitefunjia.manager.UserManager
import com.duben.infinitefunjia.mvp.model.BookContentBean
import com.duben.infinitefunjia.mvp.presenters.BookReadPresenter
import com.duben.infinitefunjia.mvp.views.BookReadView
import com.duben.infinitefunjia.read.AnimType
import com.duben.infinitefunjia.read.PageLoader
import com.duben.infinitefunjia.read.ReadChaptersAdapter
import com.duben.infinitefunjia.read.YPageView.OnCenterClickListener
import com.duben.infinitefunjia.read.models.BookModel
import com.duben.infinitefunjia.read.models.BookRecordModel
import com.duben.infinitefunjia.read.models.TxtChapterModel
import com.duben.infinitefunjia.ui.activitys.base.BaseActivity
import com.duben.infinitefunjia.ui.widgets.BookVipDialog
import com.duben.infinitefunjia.ui.widgets.DialogListener
import com.duben.infinitefunjia.ui.widgets.LongBookSettingDialog
import kotlinx.android.synthetic.main.activity_long_book_read.*
import org.litepal.LitePal
import org.litepal.crud.LitePalSupport


class LongBookReadActivity : BaseActivity(), View.OnClickListener, BookReadView {

    private val readPresenter by lazy { BookReadPresenter() }

    private var mBookModel: BookModel? = null
    private var mPageLoader: PageLoader? = null

    private var isShowSetting = true

    private var bookSettingDialog: LongBookSettingDialog? = null
    private var bookVipDialog: BookVipDialog? = null

    //目录适配器
    private var readChaptersAdapter: ReadChaptersAdapter? = null

    override fun getContentViewLayoutID() = R.layout.activity_long_book_read

    override fun isApplyKitKatTranslucency() = false

    override fun getBundleExtras(extras: Bundle?) {
        super.getBundleExtras(extras)
        extras?.let {
            mBookModel = BookModel()
            mBookModel?.bookId = extras.getString(Constant.ROUTE_BOOK_ID).toString()
        }
    }

    override fun initViewsAndEvents() {
        initWidgets()
        initListener()
        popBookSettingDialog()
        popBookVipDialog()
        readPresenter.attachView(this)
        mBookModel?.let { readPresenter.addReadRecord(it.bookId) }
    }

    override fun onResume() {
        super.onResume()
        ReadTimeCount.instance.startCountReadTime()
    }

    override fun onPause() {
        super.onPause()
        ReadTimeCount.instance.endCountReadTime()
        //保存阅读记录
        if (LitePal.where("book_id=?", (mBookModel?.id).toString())
                .findFirst(TxtChapterModel::class.java) == null
        ) {
            mPageLoader?.let { LitePal.saveAll<LitePalSupport>(it.mChapterList) }
        }

        val bookRecord = BookRecordModel()
        mPageLoader?.let {
            bookRecord.chapterPos = it.curChapterIndex
            bookRecord.pagePos = it.curPageIndex
            bookRecord.book_id = (mBookModel?.id).toString()
            bookRecord.page_color = it.bgColor
            bookRecord.text_color = it.textColor
            bookRecord.text_size = it.getmTextSize()
            bookRecord.anim_type = pageview.animType.type
            bookRecord.saveOrUpdate("book_id=?", (mBookModel?.id).toString())
        }

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_back -> finish()
            R.id.tab_setting_chapter -> {
                openChaptersCategory()

                isShowSetting = false
                view_setting.visibility = View.GONE
                tab_setting_top.visibility = View.GONE
            }
            R.id.tab_setting_book -> {
                bookSettingDialog?.show()

                isShowSetting = false
                view_setting.visibility = View.GONE
                tab_setting_top.visibility = View.GONE
            }
            R.id.tab_setting_share -> {}
            R.id.btn_submit -> {
                readyGo(VipActivity::class.java)
            }
        }
    }

    private fun popBookSettingDialog() {
        bookSettingDialog = LongBookSettingDialog(this)
        bookSettingDialog?.setOnBookSettingCallback(object :
            LongBookSettingDialog.OnLongBookSettingCallback {
            override fun onFontSize(size: Int) {
                mPageLoader?.updateTextSize(size)
                bookSettingDialog?.setSetting(null, size)
            }

            override fun onBgColor(color: Int) {
                bookSettingDialog?.setSetting(color)
                mPageLoader?.setPageStyle(color, Color.BLACK)
            }
        })
    }

    private fun popBookVipDialog() {
        bookVipDialog = BookVipDialog(this, object : DialogListener() {
            override fun onClick(dialog: Dialog?, v: View?) {
                super.onClick(dialog, v)
                dialog?.dismiss()
                readyGo(VipActivity::class.java)
            }
        })
    }

    private fun initWidgets() {
        //隐藏状态栏
        val view = window.decorView
        view.systemUiVisibility =
            (View.SYSTEM_UI_FLAG_FULLSCREEN or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_IMMERSIVE)

        //书籍
        mPageLoader = pageview.getPageLoader(mBookModel)
        pageview.animType = AnimType.ALIKE
        //从数据库加载分章信息
        mPageLoader?.mChapterList = LitePal.where("book_id=?", "" + mBookModel?.id)
            .find(TxtChapterModel::class.java)

        //获取上次的数据
        val recordModel: BookRecordModel? = LitePal.where("book_id=?", "" + mBookModel?.id)
            .findFirst(BookRecordModel::class.java)
        if (recordModel != null) {
            bookSettingDialog?.setSetting(recordModel.page_color, recordModel.text_size)

            mPageLoader?.run {
                curChapterIndex = recordModel.chapterPos
                curPageIndex = recordModel.pagePos
                setTextSize(recordModel.text_size)
                textColor = recordModel.text_color
                setPageBgColor(recordModel.page_color)

                when (recordModel.anim_type) {
                    1 -> pageview.animType = AnimType.SLIDE
                    2 -> pageview.animType = AnimType.COVER
                    3 -> pageview.animType = AnimType.ALIKE
                    else -> pageview.animType = AnimType.NONE
                }
            }
        } else {
            mPageLoader?.let {
                it.setPageBgColor(BookConstant.BOOK_CONTENT_COLOR_1.color)
                it.setTextSize(18)
                it.textColor = Color.BLACK
            }
//            pageview.animType = AnimType.SLIDE
            bookSettingDialog?.setSetting(BookConstant.BOOK_CONTENT_COLOR_1.color, 18)
        }

        pageview.setOnCenterClickListener(object : OnCenterClickListener {
            override fun centerClicked() {
                if (isShowSetting) {
                    view_setting.visibility = View.VISIBLE
                    tab_setting_top.visibility = View.VISIBLE
                } else {
                    view_setting.visibility = View.GONE
                    tab_setting_top.visibility = View.GONE
                }
                isShowSetting = !isShowSetting
            }

            override fun touPage() {
                if (isShowSetting) {
                    view_setting.visibility = View.GONE
                    tab_setting_top.visibility = View.GONE
                    isShowSetting = false
                }
            }

        })

        //初始化目录
        initChapterRecyclerView()
    }

    private fun initChapterRecyclerView() {
        read_rv_category.layoutManager = LinearLayoutManager(this)
        readChaptersAdapter = ReadChaptersAdapter(mPageLoader?.mChapterList, this) { pos ->
            mPageLoader?.let {
                if (it.mChapterList[pos].isLock && !UserManager.getInstance().vipFlag) {
                    bookVipDialog?.show()
                } else {
                    mPageLoader?.skipToChapter(pos)
                }
                read_drawer.closeDrawers()
            }
        }

        //章节回调
        mPageLoader?.setChapterChangeListener(object : PageLoader.OnChapterChangeListener {
            override fun onChapterChange(pos: Int) {
                readChaptersAdapter?.setChapter(pos)
            }

            override fun requestChapters(requestChapters: List<TxtChapterModel>) {}
            override fun finishedLoadChapters(bookChapters: List<TxtChapterModel>) {
                mPageLoader?.let {
                    tv_title.text = "阅读:${it.bookTitle}"
                    it.pageView.showCategory()
                }
                readChaptersAdapter?.setChapterModels(bookChapters)
            }

            override fun onIsVipPage() {
                bookVipDialog?.show()
            }
        })
        read_rv_category.adapter = readChaptersAdapter
    }

    //目录
    private fun openChaptersCategory() {
        mPageLoader?.let {
            read_rv_category.scrollToPosition(it.curChapterIndex)
            read_drawer.openDrawer(GravityCompat.START)
        }
    }

    private fun initListener() {
        tab_setting_chapter.setOnClickListener(this)
        tab_setting_share.setOnClickListener(this)
        tab_setting_book.setOnClickListener(this)
        iv_back.setOnClickListener(this)
    }

    override fun getBookDetailSuc(data: BookContentBean) {
    }

    override fun getBookDetailFail() {

    }

    override fun onDestroy() {
        super.onDestroy()
        readPresenter.detachView()
    }

    override fun isApplyStatusBarTranslucency(): Boolean {
        return true
    }
}
