package com.duben.infinitefunjia.ui.activitys

import android.content.Context
import android.media.AudioManager
import android.os.Bundle
import android.view.KeyEvent
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentTransaction
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.AppConfig
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.ui.activitys.base.BaseActivity
import com.duben.infinitefunjia.ui.fragment.BookCityFragment
import com.duben.infinitefunjia.ui.fragment.BookShelfFragment
import com.duben.infinitefunjia.ui.fragment.CountFragment
import com.duben.infinitefunjia.ui.fragment.MyFragment
import kotlinx.android.synthetic.main.activity_main.*
import org.litepal.LitePal

/**
 * 描述：首页
 * 作者：孟崔广
 * 时间：2024/7/3 18:06
 */
class MainActivity : BaseActivity(), View.OnClickListener {

    private var audioManager: AudioManager? = null

    // 底部标签切换的Fragment
    private var bookShelfFragment: Fragment? = null
    private var bookCityFragment: Fragment? = null
    private var countFragment: Fragment? = null
    private var myFragment: Fragment? = null
    private var currentFragment: Fragment? = null

    override fun getContentViewLayoutID(): Int {
        return R.layout.activity_main
    }

    override fun initViewsAndEvents() {
        audioManager = getSystemService(Context.AUDIO_SERVICE) as AudioManager

        //数据库 LitePal
        LitePal.initialize(this)

        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_TWO
        if (bookCityFragment == null) {
            bookCityFragment = BookCityFragment()
        }
        if (!bookCityFragment!!.isAdded) {
            // 提交事务
            supportFragmentManager.beginTransaction()
                .add(R.id.content_layout, bookCityFragment!!).commitAllowingStateLoss()

            // 记录当前Fragment
            currentFragment = bookCityFragment
        }
        tab_rl_bookshelf.setOnClickListener(this)
        tab_rl_bookcity.setOnClickListener(this)
        tab_rl_bookstore.setOnClickListener(this)
        tab_rl_my.setOnClickListener(this)
        tab_tv_bookcity.isSelected = true
        tab_iv_bookcity.isSelected = true

        initEditView()
    }

    override fun isApplyKitKatTranslucency(): Boolean {
        return false
    }

    override fun onDestroy() {
        super.onDestroy()
        currentFragment = null
        audioManager = null
    }

    public override fun onSaveInstanceState(outState: Bundle) {
        //将这一行注释掉，阻止activity保存fragment的状态，防止UI重叠
        //super.onSaveInstanceState(outState);
    }

    var oldTime: Long = 0
    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        when (keyCode) {
            KeyEvent.KEYCODE_VOLUME_UP -> {
                audioManager?.adjustStreamVolume(
                    AudioManager.STREAM_MUSIC,
                    AudioManager.ADJUST_RAISE,
                    AudioManager.FX_FOCUS_NAVIGATION_UP
                )
                return true
            }
            KeyEvent.KEYCODE_VOLUME_DOWN -> {
                audioManager?.adjustStreamVolume(
                    AudioManager.STREAM_MUSIC,
                    AudioManager.ADJUST_LOWER,
                    AudioManager.FX_FOCUS_NAVIGATION_UP
                )
                return true
            }
            KeyEvent.KEYCODE_BACK -> {
                // 设置为后台
                val currentTime = System.currentTimeMillis()
                if (currentTime - oldTime < 2 * 1000) {
                    baseApplication.exitApp()
                } else {
                    showToast("再次点击退出" + getString(R.string.app_name))
                    oldTime = currentTime
                }
            }
        }
        return true
    }

    override fun onClick(view: View) {
        when (view.id) {
            R.id.tab_rl_bookshelf -> clickTab1Layout()
            R.id.tab_rl_bookcity -> clickTab2Layout()
            R.id.tab_rl_bookstore -> clickTab3Layout()
            R.id.tab_rl_my -> clickTab4Layout()
        }
    }

    fun clickTab1Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_ONE
        if (bookShelfFragment == null) {
            bookShelfFragment = BookShelfFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), bookShelfFragment!!)
        tab_tv_bookshelf.isSelected = true
        tab_iv_bookshelf.isSelected = true
        tab_tv_bookcity.isSelected = false
        tab_iv_bookcity.isSelected = false
        tab_tv_bookstore.isSelected = false
        tab_iv_bookstore.isSelected = false
        tab_tv_my.isSelected = false
        tab_iv_my.isSelected = false
    }

    fun clickTab2Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_TWO
        if (bookCityFragment == null) {
            bookCityFragment = BookCityFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), bookCityFragment!!)
        tab_tv_bookshelf.isSelected = false
        tab_iv_bookshelf.isSelected = false
        tab_tv_bookcity.isSelected = true
        tab_iv_bookcity.isSelected = true
        tab_tv_bookstore.isSelected = false
        tab_iv_bookstore.isSelected = false
        tab_tv_my.isSelected = false
        tab_iv_my.isSelected = false
    }

    fun clickTab3Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_THREE
        if (countFragment == null) {
            countFragment = CountFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), countFragment!!)
        tab_tv_bookshelf.isSelected = false
        tab_iv_bookshelf.isSelected = false
        tab_tv_bookcity.isSelected = false
        tab_iv_bookcity.isSelected = false
        tab_tv_bookstore.isSelected = true
        tab_iv_bookstore.isSelected = true
        tab_tv_my.isSelected = false
        tab_iv_my.isSelected = false
    }

    fun clickTab4Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_FOUR
        if (myFragment == null) {
            myFragment = MyFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), myFragment!!)
        tab_tv_bookshelf.isSelected = false
        tab_iv_bookshelf.isSelected = false
        tab_tv_bookcity.isSelected = false
        tab_iv_bookcity.isSelected = false
        tab_tv_bookstore.isSelected = false
        tab_iv_bookstore.isSelected = false
        tab_tv_my.isSelected = true
        tab_iv_my.isSelected = true
    }

    private fun addOrShowFragment(
        transaction: FragmentTransaction,
        fragment: Fragment
    ) {
        if (currentFragment === fragment) return
        if (!fragment.isAdded) { // 如果当前fragment未被添加，则添加到Fragment管理器中
            transaction.hide(currentFragment!!)
                .add(R.id.content_layout, fragment).commitAllowingStateLoss()
        } else {
            transaction.hide(currentFragment!!).show(fragment).commitAllowingStateLoss()
        }
        currentFragment = fragment
    }

    private fun initEditView() {
        tv_full.setOnClickListener {
            bookShelfFragment?.apply {
                (this as BookShelfFragment).fullChoice()
            }
        }
        tv_delete.setOnClickListener {
            bookShelfFragment?.apply {
                (this as BookShelfFragment).delete()
            }
        }
    }

    fun showEdit(isEdit: Boolean) {
        if (isEdit) {
            ll_edit.visibility = View.VISIBLE
        } else {
            ll_edit.visibility = View.GONE
        }
    }

}