package com.duben.infinitefunjia.ui.activitys

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.TextPaint
import android.text.style.ClickableSpan
import android.view.KeyEvent
import android.view.View
import androidx.core.content.ContextCompat
import cn.sharesdk.framework.ShareSDK
import cn.sharesdk.wechat.friends.Wechat
import com.daimajia.androidanimations.library.Techniques
import com.daimajia.androidanimations.library.YoYo
import com.duben.infinitefunjia.MintsApplication
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.login.LoginApi
import com.duben.infinitefunjia.login.OnLoginListener
import com.duben.infinitefunjia.mvp.model.WXInfo
import com.duben.infinitefunjia.mvp.presenters.LoginPresenter
import com.duben.infinitefunjia.mvp.views.LoginView
import com.duben.infinitefunjia.ui.activitys.base.BaseActivity
import com.duben.infinitefunjia.utils.CommonUtils
import com.duben.infinitefunjia.utils.SpanUtils
import com.duben.infinitefunjia.utils.ToastUtil
import com.duben.library.utils.json.JsonUtil
import com.duben.library.utils.nodoubleclick.AntiShake
import kotlinx.android.synthetic.main.activity_wx_login.*


/**
 * 描述：微信登录
 * 作者：孟崔广
 * 时间：2022/12/2 13:50
 */
class WxLoginActivity : BaseActivity(), LoginView, View.OnClickListener, OnLoginListener {

    private val loginPresenter by lazy { LoginPresenter() }
    private val loginApi by lazy { LoginApi() }

    private var wxInfo: WXInfo? = null
    private var isCheckAgree = false

    override fun getContentViewLayoutID() = R.layout.activity_wx_login

    override fun toggleOverridePendingTransition() = true

    override fun getOverridePendingTransitionMode() = TransitionMode.BOTTOM

    override fun isApplyKitKatTranslucency() = false

    override fun initViewsAndEvents() {
        loginPresenter.attachView(this)
        initView()
        initListener()
    }

    override fun onDestroy() {
        super.onDestroy()
        loginApi.setOnLoginListener(null)
        loginPresenter.detachView()
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        return if (keyCode == KeyEvent.KEYCODE_BACK) {
            true
        } else super.onKeyDown(keyCode, event)
    }

    override fun finish() {
        super.finish()
        //关闭窗体动画显示
        overridePendingTransition(0, R.anim.push_bottom_out)
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
            R.id.ivLoginBack -> {
                finish()
            }
            R.id.tvLoginMobile -> {
                readyGo(MobileLoginActivity::class.java)
            }
            R.id.rlLoginWx -> {
                if (!isCheckAgree) {
                    ToastUtil.showLong(this,"请勾选同意后再进行登录")
                    llWxloginCheck.postDelayed({
                        YoYo.with(Techniques.Shake).duration(500).repeat(2).playOn(llWxloginCheck)
                    }, 200)
                    return
                }

                val wechat = ShareSDK.getPlatform(Wechat.NAME)
                if (!wechat.isClientValid) {
                    showToast("请先安装微信")
                    return
                }

                loginApi.setOnLoginListener(this@WxLoginActivity)
                loginApi.setPlatform(Wechat.NAME)
                loginApi.login(this@WxLoginActivity)
            }
        }
    }

    override fun loginSuc() {
        if(isFinishing) return

        showToast("登录成功")

        hideLoading()

        Handler(Looper.getMainLooper()).postDelayed({
            readyGoThenKill(MainActivity::class.java)
        },1000)

    }

    override fun sendCodeSuc() {

    }

    override fun mergeAccount(mobile: String, wxOpenId: String, key: String) {

    }

    override fun onLogin(platform: String?, res: HashMap<String, Any>): Boolean {
        try {
            wxInfo = WXInfo()
            wxInfo?.let {
                it.unionid = res["unionid"].toString() + ""
                it.openid = res["openid"].toString() + ""
                it.province = res["province"].toString() + ""
                it.city = res["city"].toString() + ""
                it.country = res["country"].toString() + ""
                it.headimgurl = res["headimgurl"].toString() + ""
                it.nickname = res["nickname"].toString() + ""
                it.sex = (res["sex"].toString() + "").toInt()
            }

            if (!isFinishing) {
                loginPresenter.login(JsonUtil.toJson(wxInfo))
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    private fun initView() {

        SpanUtils.with(tvLoginAgreement)
            .append("已阅读并同意")
            .append("《用户注册协议》").setClickSpan(object : ClickableSpan() {
                override fun onClick(widget: View) {
                    val bundle = Bundle()
                    bundle.putString(WebActivity.WEB_TITLE, getString(R.string.register_name))
                    bundle.putString(WebActivity.WEB_URL, Constant.REGISTER_URL)
                    readyGo(WebActivity::class.java, bundle)
                }

                override fun updateDrawState(ds: TextPaint) {
                    ds.color = ContextCompat.getColor(mContext, R.color.main_mints)
                    ds.isUnderlineText = false
                }
            })
            .append("、")
            .append("《用户隐私协议》").setClickSpan(object : ClickableSpan() {
                override fun onClick(widget: View) {
                    val bundle = Bundle()
                    bundle.putString(WebActivity.WEB_TITLE, getString(R.string.privacy_name))
                    bundle.putString(WebActivity.WEB_URL, Constant.PRIVACY_URL)
                    readyGo(WebActivity::class.java, bundle)
                }

                override fun updateDrawState(ds: TextPaint) {
                    ds.color = ContextCompat.getColor(mContext, R.color.main_mints)
                    ds.isUnderlineText = false
                }
            })
            .append("与您的利益切身相关。请您注册前务必仔细阅读!")
            .create()

    }

    private fun initListener() {
        ivLoginBack.setOnClickListener(this)
        tvLoginMobile.setOnClickListener(this)
        rlLoginWx.setOnClickListener(this)

        wxloginCheck.setOnCheckedChangeListener { buttonView, isChecked ->
            this.isCheckAgree = isChecked
        }
    }
}

