package com.duben.infinitefunjia.ui.adapter

import android.app.Activity
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.view.View
import android.widget.CheckBox
import android.widget.ImageView
import android.widget.ImageView.ScaleType
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.mvp.model.BookBean
import com.duben.infinitefunjia.utils.RoundBackgroundColorSpan
import com.duben.library.utils.GlideUtils

/**
 *  @author Assen
 *  @date 2023/7/5
 *  @desc
 */
class BookShelfAdapter(var activity: Activity) :
    BaseQuickAdapter<BookBean, BaseViewHolder>(R.layout.item_bookshelf) {

    private var showEdit = false

    override fun convert(holder: BaseViewHolder, item: BookBean) {
        if (holder.adapterPosition == data.size - 1) {
            holder.getView<ImageView>(R.id.image_iv).scaleType = ScaleType.FIT_XY
            return
        }

        holder.getView<CheckBox>(R.id.cb).isChecked = item.isChecked
        holder.getView<CheckBox>(R.id.cb).setOnCheckedChangeListener { _, isChecked ->
            item.isChecked = isChecked
        }
        holder.getView<View>(R.id.fm_mask).visibility = if (showEdit) View.VISIBLE else View.GONE

        holder.getView<TextView>(R.id.title_tv).text = item.title
        if (item.tagList != null && item.tagList.size > 0) {
            val span = SpannableStringBuilder()
            for (i in 0 until item.tagList.size) {
                val labelText = SpannableString("  " + item.tagList[i].name + "    ")
                val start = 0
                val end = item.tagList[i].name.length + 4

                labelText.setSpan(
                    RoundBackgroundColorSpan(
                        ContextCompat.getColor(activity, R.color.color_FEF3F3),
                        ContextCompat.getColor(activity, R.color.color_F1413C)
                    ), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE
                )

                span.append(labelText)
            }
            holder.getView<TextView>(R.id.label_tv).text = span
        }
        GlideUtils.loadImageViewNoAnim2(context, item.avatar, holder.getView(R.id.image_iv))
    }

    // 设置编辑状态
    fun setEditStyle(showEdit: Boolean) {
        this.showEdit = showEdit
        // 清除选中状态
        if (!this.showEdit) {
            for (datum in data) {
                datum.isChecked = false
            }
        }
        notifyDataSetChanged()
    }

}