package com.duben.infinitefunjia.ui.adapter

import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.airbnb.lottie.LottieAnimationView
import com.airbnb.lottie.LottieComposition
import com.airbnb.lottie.LottieCompositionFactory
import com.airbnb.lottie.LottieDrawable
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.mvp.model.BookBean

class WatchRecordAdapter :
    BaseQuickAdapter<BookBean, BaseViewHolder>(R.layout.item_watch_record) {

    override fun convert(holder: BaseViewHolder, item: BookBean) {

        holder.getView<TextView>(R.id.tv_watch_title).text = item.title
        holder.getView<TextView>(R.id.tv_watch_text).text = item.summary
        holder.getView<TextView>(R.id.tv_watch_user).text = "作者:" + item.author
        if (item.isCollect) {
            holder.getView<ImageView>(R.id.iv_collect).setImageResource(R.mipmap.home_collect_img_1)
        } else {
            holder.getView<ImageView>(R.id.iv_collect).setImageResource(R.mipmap.home_collect_img_0)
        }

        holder.getView<View>(R.id.ll_collect).setOnClickListener {
            if (!item.isCollect) {
                playCollectAnim(holder.getView(R.id.iv_collect))
                mOnItemCollectClickListener?.onItemCollectClick(item)
            } else {
                playCancelCollectAnim(holder.getView(R.id.iv_collect))
                mOnItemCollectClickListener?.onItemCollectClick(item)
            }
        }
    }


    private fun playCollectAnim(view: LottieAnimationView) {
        val lottieDrawable = LottieDrawable()
        LottieCompositionFactory.fromAsset(context, "home_collect.json")
            .addListener { result: LottieComposition? ->
                lottieDrawable.setImagesAssetsFolder("images/")
                lottieDrawable.composition = result
                lottieDrawable.loop(false)
                lottieDrawable.playAnimation()
            }
        view.setImageDrawable(lottieDrawable)
    }

    private fun playCancelCollectAnim(view: LottieAnimationView) {
        val lottieDrawable = LottieDrawable()
        LottieCompositionFactory.fromAsset(context, "home_cancel_collect.json")
            .addListener { result: LottieComposition? ->
                lottieDrawable.setImagesAssetsFolder("images/")
                lottieDrawable.composition = result
                lottieDrawable.loop(false)
                lottieDrawable.playAnimation()
            }
        view.setImageDrawable(lottieDrawable)
    }

    private var mOnItemCollectClickListener: OnItemCollectClickListener? = null

    fun setOnItemCollectClickListener(onItemCollectClickListener: OnItemCollectClickListener) {
        this.mOnItemCollectClickListener = onItemCollectClickListener
    }

    interface OnItemCollectClickListener {
        fun onItemCollectClick(bean: BookBean)
    }

}