package com.duben.infinitefunjia.ui.fragment

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.GridLayoutManager
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.ad.AdManager
import com.duben.infinitefunjia.common.AppConfig
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.BookReadManager
import com.duben.infinitefunjia.manager.UserManager
import com.duben.infinitefunjia.mvp.model.BookBean
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.model.FirstBookBean
import com.duben.infinitefunjia.mvp.presenters.BookCityPresenter
import com.duben.infinitefunjia.mvp.views.BookCityView
import com.duben.infinitefunjia.ui.activitys.BookSearchActivity
import com.duben.infinitefunjia.ui.activitys.VipActivity
import com.duben.infinitefunjia.ui.adapter.HotBookAdapter
import com.duben.infinitefunjia.ui.adapter.NewBookAdapter
import com.duben.infinitefunjia.ui.adapter.ImageTitleAdapter
import com.duben.infinitefunjia.ui.fragment.base.LazyLoadBaseFragment
import com.duben.infinitefunjia.utils.AppPreferencesManager
import com.duben.infinitefunjia.utils.GridSpacingItemDecoration
import com.duben.infinitefunjia.utils.RoundBackgroundColorSpan
import com.duben.library.utils.GlideUtils
import com.duben.library.utils.nodoubleclick.AntiShake
import kotlinx.android.synthetic.main.fragment_main_bookcity.*

/**
 * 书城-2
 */
class BookCityFragment : LazyLoadBaseFragment(), BookCityView, View.OnClickListener {

    private val bookCityPresenter by lazy { BookCityPresenter() }
    private val userManager by lazy { UserManager.getInstance() }

    private var hotBookAdapter: HotBookAdapter? = null
    private var hotBookData = arrayListOf<BookBean>()
    private var newBookAdapter: NewBookAdapter? = null
    private var newBookData = arrayListOf<BookBean>()

    override fun getContentViewLayoutID() = R.layout.fragment_main_bookcity

    override fun initViewsAndEvents() {
        bookCityPresenter.attachView(this)

        if (!UserManager.getInstance().vipFlag) {
            AdManager.instance.preLoadAd(requireActivity())
        }

        initView()
        initListener()

        Handler(Looper.getMainLooper()).postDelayed({
            goToMainActivity()
        }, 300)
    }

    private fun initView() {
        initHotAdapter()
        initNewAdapter()
    }

    private fun initListener() {
        iv_change_hot.setOnClickListener(this)
        iv_change_new.setOnClickListener(this)
        view_search.setOnClickListener(this)
    }

    private fun initHotAdapter() {
        hotBookAdapter = HotBookAdapter(requireActivity())
        hotBookAdapter?.setOnItemClickListener { _, _, position ->
            BookReadManager.instance.goReadPage(requireActivity(), hotBookData[position])
        }
        rv_hotbook.layoutManager = GridLayoutManager(requireContext(), 4)
        val itemDecoration = GridSpacingItemDecoration(5, 10, false)
        rv_hotbook.addItemDecoration(itemDecoration)
        rv_hotbook.adapter = hotBookAdapter
    }

    private fun getHotHeader(headData: BookBean): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_hotbook, null)
        view.findViewById<TextView>(R.id.title_tv).text = headData.title
        view.findViewById<TextView>(R.id.info_tv).text = headData.summary
        GlideUtils.loadImageViewNoAnim2(context, headData.avatar, view.findViewById(R.id.image_iv))
        if (headData.tagList != null && headData.tagList.size > 0) {
            val span = SpannableStringBuilder()
            for (i in 0 until headData.tagList.size) {
                val labelText = SpannableString("  " + headData.tagList[i].name + "    ")
                val start = 0
                val end = headData.tagList[i].name.length + 4
                labelText.setSpan(
                    RoundBackgroundColorSpan(
                        ContextCompat.getColor(requireContext(), R.color.color_FEF3F3),
                        ContextCompat.getColor(requireContext(), R.color.color_F1413C)
                    ), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE
                )
                span.append(labelText)
            }
            view.findViewById<TextView>(R.id.label_tv).text = span
        }
        view.setOnClickListener {
            val bundle = Bundle()
            bundle.putString(Constant.ROUTE_BOOK_ID, headData.id)
            BookReadManager.instance.goReadPage(requireActivity(), headData)
        }
        return view
    }

    private fun initNewAdapter() {
        newBookAdapter = NewBookAdapter(requireActivity())
        newBookAdapter?.setOnItemClickListener { _, _, position ->
            BookReadManager.instance.goReadPage(requireActivity(), newBookData[position])
        }
        rv_newbook.layoutManager = GridLayoutManager(requireContext(), 2)
        rv_newbook.adapter = newBookAdapter
    }

    private fun goToMainActivity() {
        bookCityPresenter.getSoltVedio()
    }

    override fun onHiddenChanged(hidden: Boolean) {
        if (hidden) { // 不在最前端界面显示
            onPause()
        } else { // 重新显示到最前端中
            onResume()
        }
        super.onHiddenChanged(hidden)
    }

    override fun onFragmentFirstVisible() {
        super.onFragmentFirstVisible()
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_TWO) {
            if (AntiShake.check(banner?.id)) return
            banner?.start()
            if (TextUtils.isEmpty(userManager?.userID)) {
                //获取游客登录数据
                bookCityPresenter.userLogin()
            } else {
                bookCityPresenter.getBannerBook()
                bookCityPresenter.getHotBook()
                bookCityPresenter.getNewBook()
            }
        }
    }

    override fun getHotBookSuc(data: BookListBean) {
        hotBookData.clear()
        hotBookData.addAll(data.records)
        hotBookData.removeAt(0)

        hotBookAdapter?.let {
            it.setHeaderView(getHotHeader(data.records[0]))
            it.setList(hotBookData)
        }
    }

    override fun getHotBookFail() {}

    override fun getNewBookSuc(data: BookListBean) {
        newBookData.clear()
        newBookData.addAll(data.records)
        newBookAdapter?.setList(newBookData)
    }

    override fun getNewBookFail() {}

    override fun getBannerBookSuc(data: BookListBean) {
        banner.addBannerLifecycleObserver(this)
            .setAdapter(ImageTitleAdapter(data.records))
            .setOnBannerListener { data2, _ ->
                BookReadManager.instance.goReadPage(requireActivity(), data2 as BookBean)
            }
    }

    override fun getBannerBookFail() {}
    override fun getSoltVedioSuc(data: FirstBookBean) {
        val firstVedio = AppPreferencesManager.get().getBoolean(Constant.IS_FIRST_VEDIO, true)
        if (data.articleMsg != null && firstVedio) {
            BookReadManager.instance.goReadPage(requireActivity(), data.articleMsg)
        } else {
            if (!UserManager.getInstance().vipFlag) {
                val bundle = Bundle()
                bundle.putBoolean(VipActivity.IS_MAIN, true)
                readyGo(VipActivity::class.java, bundle)
            }
        }
    }

    override fun onClick(v: View?) {
        v?.let {
            when (it.id) {
                R.id.iv_change_hot -> {
                    bookCityPresenter.getHotBook()
                }
                R.id.iv_change_new -> {
                    bookCityPresenter.getNewBook()
                }
                R.id.view_search -> {
                    readyGo(BookSearchActivity::class.java)
                }
            }
        }
    }
}