package com.duben.infinitefunjia.ui.fragment

import android.os.Bundle
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.ad.express.MainExpressManager
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.BookReadManager
import com.duben.infinitefunjia.mvp.model.BookBean
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.presenters.PicContentPresenter
import com.duben.infinitefunjia.mvp.views.PicContentView
import com.duben.infinitefunjia.ui.adapter.CountAdapter
import com.duben.infinitefunjia.ui.fragment.base.LazyLoadBaseFragment
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import kotlinx.android.synthetic.main.fragment_movie.*

/**
 * 书库-列表
 */
class BookListFragment : LazyLoadBaseFragment(), PicContentView, OnRefreshListener,
    OnLoadMoreListener {

    private var mType = 0
    private var mPosition = 0 // 首页-第一个tab-第一个recyclerview Item
    private var hotBookData: MutableList<BookBean> = mutableListOf()

    companion object {
        private const val VIDEO_TYPE = "VIDEO_TYPE"
        private const val VIDEO_POSITION = "VIDEO_POSITION"

        fun newInstance(type: Int, position: Int): Fragment {
            val args = Bundle()
            args.putInt(VIDEO_TYPE, type)
            args.putInt(VIDEO_POSITION, position)
            val fragment = BookListFragment()
            fragment.arguments = args
            return fragment
        }
    }

    private val picContentPresenter by lazy { PicContentPresenter() }

    private var hotPage = 1 // 分页
    private var hotPageSize = Constant.PAGE_SIZE // 分页

    private lateinit var hotStyleAdapter: CountAdapter

    override fun getContentViewLayoutID() = R.layout.fragment_movie

    override fun initViewsAndEvents() {
        arguments?.let {
            mType = it.getInt(VIDEO_TYPE, mType)
            mPosition = it.getInt(VIDEO_POSITION, mPosition)
        }

        picContentPresenter.attachView(this)

        initView()
        initListener()


        hotPage = 1
        srlMainPage.resetNoMoreData()
        picContentPresenter.getHomePageImageList(mType, hotPage, hotPageSize)
    }

    override fun onResume() {
        super.onResume()


    }

    override fun onDestroy() {
        super.onDestroy()
        picContentPresenter.detachView()
    }


    override fun onLoadMore(refreshLayout: RefreshLayout) {
        hotPage = ++hotPage
        picContentPresenter.getHomePageImageList(mType, hotPage, hotPageSize)
    }

    override fun getHomePageImageListSuc(data: BookListBean) {
        if (activity == null || requireActivity().isFinishing) {
            srlMainPage?.finishRefresh(true)
            return
        }

        if (::hotStyleAdapter.isInitialized) {
            if (hotPage == 1) {
                srlMainPage.finishRefresh(true)
                hotStyleAdapter.setNewInstance(data.records)
            } else {
                if (data.records.size < hotPageSize) {
                    srlMainPage.finishLoadMoreWithNoMoreData()
                } else {
                    srlMainPage.finishLoadMore()
                }
                hotStyleAdapter.addData(data.records)
            }
        }
    }

    override fun getHomePageImageListFail() {
        hotPage = 1
        srlMainPage?.finishRefresh(false)
    }

    private fun initView() {
        rv_pic_list.layoutManager = LinearLayoutManager(context)
        hotStyleAdapter = CountAdapter()
        hotStyleAdapter.animationEnable = false
        rv_pic_list.adapter = hotStyleAdapter
        hotStyleAdapter.setOnCountItemClickListener(object :
            CountAdapter.OnCountItemClickListener {
            override fun onItemClick(bean: BookBean) {
                BookReadManager.instance.goReadPage(requireActivity(), bean)
            }
        })
    }

    private fun initListener() {
        srlMainPage.setOnRefreshListener(this)
        srlMainPage.setOnLoadMoreListener(this)
//        srlMainPage.autoRefresh()
    }

    override fun onDestroyView() {
        super.onDestroyView()
        MainExpressManager.instance.destroy()
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        hotPage = 1
        srlMainPage.resetNoMoreData()
        picContentPresenter.getHomePageImageList(mType, hotPage, hotPageSize)
    }

}