package com.duben.infinitefunjia.ui.fragment

import android.view.View
import androidx.recyclerview.widget.GridLayoutManager
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.AppConfig
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.BookReadManager
import com.duben.infinitefunjia.manager.ReadTimeCount
import com.duben.infinitefunjia.mvp.model.BookBean
import com.duben.infinitefunjia.mvp.model.BookListBean
import com.duben.infinitefunjia.mvp.presenters.BookShelfPresenter
import com.duben.infinitefunjia.mvp.views.BookShelfView
import com.duben.infinitefunjia.ui.activitys.MainActivity
import com.duben.infinitefunjia.ui.adapter.BookShelfAdapter
import com.duben.infinitefunjia.ui.adapter.LikeBookAdapter
import com.duben.infinitefunjia.ui.fragment.base.LazyLoadBaseFragment
import com.duben.infinitefunjia.utils.GridSpacingItemDecoration
import com.duben.infinitefunjia.utils.SpanUtils
import kotlinx.android.synthetic.main.fragment_main_bookshelf.*


class BookShelfFragment : LazyLoadBaseFragment(), BookShelfView, View.OnClickListener {

    private val bookShelfPresenter by lazy { BookShelfPresenter() }
    private val shelfBookData = arrayListOf<BookBean>()
    private var bookShelfAdapter: BookShelfAdapter? = null

    private var likeBookAdapter: LikeBookAdapter? = null
    private var likeBookData = arrayListOf<BookBean>()

    override fun getContentViewLayoutID() = R.layout.fragment_main_bookshelf

    override fun initViewsAndEvents() {
        bookShelfPresenter.attachView(this)

        initListener()
        initLikeAdapter()
        initShelfAdapter()
    }

    private fun initShelfAdapter() {
        bookShelfAdapter = BookShelfAdapter(requireActivity())
        bookShelfAdapter?.setOnItemClickListener { _, _, position ->
            if (position >= shelfBookData.size - 1) {
                (requireActivity() as MainActivity).clickTab2Layout()
                return@setOnItemClickListener
            }

            BookReadManager.instance.goReadPage(requireActivity(),shelfBookData[position])
        }
        rv_bookshelf.layoutManager = GridLayoutManager(requireContext(), 3)
        val itemDecoration = GridSpacingItemDecoration(3, 10, false)
        rv_bookshelf.addItemDecoration(itemDecoration)
        rv_bookshelf.adapter = bookShelfAdapter
    }

    private fun initLikeAdapter() {
        likeBookAdapter = LikeBookAdapter(requireActivity())
        likeBookAdapter?.setOnItemClickListener { _, _, position ->
            BookReadManager.instance.goReadPage(requireActivity(),likeBookData[position])
        }
        rv_booklike.layoutManager = GridLayoutManager(requireContext(), 4)
        val itemDecoration = GridSpacingItemDecoration(4, 10, false)
        rv_booklike.addItemDecoration(itemDecoration)
        rv_booklike.adapter = likeBookAdapter
    }

    override fun onFragmentResume() {
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_ONE) {
            bookShelfPresenter.getCollectList()
            bookShelfPresenter.getArticleLike()

            getReadTime()
        }
    }

    private fun getReadTime() {
        val time = ReadTimeCount.instance.getReadTime()
        tv_read_hour.text = SpanUtils()
            .append(time[0])
            .setFontSize(20, true)
            .setBold()
            .append("\n小时")
            .setFontSize(10, true)
            .create()
        tv_read_minute.text = SpanUtils()
            .append(time[1])
            .setFontSize(20, true)
            .setBold()
            .append("\n分")
            .setFontSize(10, true)
            .create()
        tv_read_second.text = SpanUtils()
            .append(time[2])
            .setFontSize(20, true)
            .setBold()
            .append("\n秒")
            .setFontSize(10, true)
            .create()
    }

    override fun onHiddenChanged(hidden: Boolean) {
        if (hidden) { // 不在最前端界面显示
            onPause()
        } else { // 重新显示到最前端中
            onResume()
        }
        super.onHiddenChanged(hidden)
    }

    override fun onDestroy() {
        super.onDestroy()
        bookShelfPresenter.detachView()
    }

    private fun initListener() {
        iv_edit.setOnClickListener(this)
        iv_close.setOnClickListener(this)
        iv_change.setOnClickListener(this)
        btn_empty.setOnClickListener(this)
    }

    override fun getCollectListSuc(data: BookListBean) {
        if (data.records.size > 0) {
            bookshelf_empty.visibility = View.GONE
            rv_bookshelf.visibility = View.VISIBLE

            shelfBookData.clear()
            shelfBookData.addAll(data.records)
            shelfBookData.add(BookBean())
            bookShelfAdapter?.setList(shelfBookData)
            bookShelfAdapter?.setEditStyle(false)
        } else {
            bookshelf_empty.visibility = View.VISIBLE
            rv_bookshelf.visibility = View.GONE
        }
    }

    override fun getCollectListFail() {
    }

    override fun getArticleLikeSuc(data: BookListBean) {
        likeBookData.clear()
        likeBookData.addAll(data.records)
        likeBookAdapter?.setList(likeBookData)
    }

    override fun getArticleLikeFail() {

    }

    override fun cancelCollectSuc() {
        bookShelfPresenter.getCollectList()
        handleEditStyle(false)
    }

    override fun cancelCollectFail() {
        handleEditStyle(false)
    }

    private fun handleEditStyle(isEdit: Boolean) {
        if (isEdit) {
            iv_edit.visibility = View.GONE
            iv_close.visibility = View.VISIBLE
        } else {
            iv_edit.visibility = View.VISIBLE
            iv_close.visibility = View.GONE
        }
        (requireActivity() as MainActivity).showEdit(isEdit)
        bookShelfAdapter?.setEditStyle(isEdit)
    }

    // 全选
    fun fullChoice() {
        for (datum in shelfBookData) {
            datum.isChecked = true
        }
        bookShelfAdapter?.notifyDataSetChanged()
    }

    fun delete() {
        if (shelfBookData.size > 0) {
            var strArr = ""
            for (datum in shelfBookData) {
                if (datum.isChecked && datum.id != null) {
                    strArr += (datum.id + ",")
                }
            }
            bookShelfPresenter.cancelCollect(strArr)
        }
    }

    override fun onClick(v: View?) {
        v?.let {
            when (it.id) {
                R.id.iv_edit -> {
                    handleEditStyle(true)
                }
                R.id.iv_close -> {
                    handleEditStyle(false)
                }
                R.id.iv_change -> {
                    bookShelfPresenter.getArticleLike()
                }
                R.id.btn_empty -> {
                    (requireActivity() as MainActivity).clickTab2Layout()
                }
            }
        }
    }
}