package com.duben.infinitefunjia.ui.fragment

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.AppConfig
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.UserManager
import com.duben.infinitefunjia.mvp.model.*
import com.duben.infinitefunjia.mvp.presenters.CountPresenter
import com.duben.infinitefunjia.mvp.views.CountView
import com.duben.infinitefunjia.ui.adapter.CountAdapter
import com.duben.infinitefunjia.ui.adapter.HomeVideoPageAdapter
import com.duben.infinitefunjia.ui.fragment.base.LazyLoadBaseFragment
import com.duben.library.utils.nodoubleclick.AntiShake
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import kotlinx.android.synthetic.main.fragment_main_count.*
import java.lang.reflect.Field
import java.util.*
import kotlin.concurrent.schedule

/**
 * 书库
 */
class CountFragment : LazyLoadBaseFragment(), CountView {

    companion object {
        var mSelectTabIndex = -1
    }

    private val countPresenter by lazy { CountPresenter() }
    private val userManager by lazy { UserManager.getInstance() }

    private val tabsData = mutableListOf<BookTypeBean>()
    private var vpAdapter: HomeVideoPageAdapter? = null

    override fun getContentViewLayoutID() = R.layout.fragment_main_count

    override fun initViewsAndEvents() {
        countPresenter.attachView(this)
    }

    private fun loadData() {
        Timer().schedule(200) {
            requireActivity().runOnUiThread {
                if (AntiShake.check(tablayout?.id)) return@runOnUiThread

                if (!TextUtils.isEmpty(userManager?.userID)) {
                    countPresenter.getHomeV1Types()
                }
            }
        }
    }


    override fun onFragmentFirstVisible() {
        super.onFragmentFirstVisible()

        initVp()
        loadData()
    }

    override fun onDestroy() {
        super.onDestroy()
        countPresenter.detachView()
    }

    override fun onFragmentResume() {
        super.onFragmentResume()
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_THREE) {
            vpAdapter?.getFragments()?.get(mSelectTabIndex)?.onResume()
        }
    }
    override fun onPause() {
        super.onPause()
        vpAdapter?.getFragments()?.get(mSelectTabIndex)?.onPause()
    }

    override fun getHomeV1MsgSuc(data: HotBookTypeList) {
        tabsData.clear()
        tabsData.addAll(data.records)
        if (tabsData.isNotEmpty()) {
            mSelectTabIndex = 0
            vpAdapter?.notifyDataSetChanged()
        }
    }

    private fun initVp() {
        vpAdapter = HomeVideoPageAdapter(tabsData, this)
        vp2.adapter = vpAdapter
        vp2.desensitization()
        TabLayoutMediator(tablayout, vp2) { tab, position ->
            // 初始化Tab
            tab.id = position
            if (tabsData.size > 0) {
                tab.customView = getTabView(tabsData[position].name)
            }
        }.attach()
        tablayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                updateTab(tab, true)
                mSelectTabIndex = tab!!.id
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                updateTab(tab, false)
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
        vp2.offscreenPageLimit = 100
    }

    private fun updateTab(tab: TabLayout.Tab?, isSelected: Boolean) {
        tab?.customView?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            if (isSelected) {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.white))
                text.background=ContextCompat.getDrawable(requireContext(), R.drawable.shape_book_text_enable)
            } else {
                text.background=ContextCompat.getDrawable(requireContext(), R.drawable.shape_book_text_none)
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_6B6B6B))
            }
        }
    }

    private fun getTabView(text: String): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_video_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = text
        return view
    }

    fun ViewPager2.desensitization() {
        //动态设置ViewPager2 灵敏度
        try {
            val recyclerViewField: Field = ViewPager2::class.java.getDeclaredField("mRecyclerView")
            recyclerViewField.isAccessible = true
            val recyclerView = recyclerViewField.get(this) as RecyclerView
            val touchSlopField: Field = RecyclerView::class.java.getDeclaredField("mTouchSlop")
            touchSlopField.isAccessible = true
            val touchSlop = touchSlopField.get(recyclerView) as Int
            touchSlopField.set(recyclerView, touchSlop * 3) //6 is empirical value
        } catch (ignore: java.lang.Exception) {
        }
    }
}