package com.duben.infinitefunjia.ui.fragment

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.AppConfig
import com.duben.infinitefunjia.common.Constant
import com.duben.infinitefunjia.manager.AppHttpManager
import com.duben.infinitefunjia.manager.UserManager
import com.duben.infinitefunjia.mvp.model.BaseResponse
import com.duben.infinitefunjia.mvp.model.OrderRecordBean
import com.duben.infinitefunjia.mvp.model.UserBean
import com.duben.infinitefunjia.mvp.presenters.MyPresenter
import com.duben.infinitefunjia.mvp.views.MyView
import com.duben.infinitefunjia.ui.activitys.*
import com.duben.infinitefunjia.ui.fragment.base.LazyLoadBaseFragment
import com.duben.infinitefunjia.ui.widgets.DialogListener
import com.duben.infinitefunjia.ui.widgets.PhoneDialog
import com.duben.infinitefunjia.utils.TimeRender
import com.duben.library.net.neterror.BaseSubscriber
import com.duben.library.net.neterror.Throwable
import com.duben.library.utils.nodoubleclick.AntiShake
import kotlinx.android.synthetic.main.fragment_main_my.*

/**
 * 描述：我
 * 作者：孟崔广
 * 时间：2024/9/20 11:30
 */
class MyFragment : LazyLoadBaseFragment(), MyView, View.OnClickListener {

    private val myPresenter by lazy { MyPresenter() }
    private val userManager by lazy { UserManager.getInstance() }
    private var phoneDialog: PhoneDialog? = null
    private var mAgreementDTO: OrderRecordBean.AgreementDTO? = null

    override fun getContentViewLayoutID() = R.layout.fragment_main_my

    override fun initViewsAndEvents() {
        myPresenter.attachView(this)

        initListener()
    }

    override fun onFragmentFirstVisible() {
        super.onFragmentFirstVisible()

    }

    override fun onDestroy() {
        super.onDestroy()
        myPresenter.detachView()
    }

    override fun onFragmentResume() {
        super.onFragmentResume()

        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_FOUR) {
            if (AntiShake.check(ll_my_cancel?.id)) return

            if (!TextUtils.isEmpty(userManager?.userID)) {
                myPresenter.getMyInfo()
            } else {
                myPresenter.userLogin()
            }
        }
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
            R.id.tv_my_account, R.id.ll_my_login -> {
                if (!userManager.userIsLogin()) {
                    readyGo(MobileLoginActivity::class.java)
                }
            }
            R.id.tv_my_btn -> {
                readyGo(VipActivity::class.java)
            }
            R.id.ll_my_history -> {
                readyGo(WatchRecordActivity::class.java)
            }
            R.id.ll_my_change -> {
                readyGo(ChangeVipActivity::class.java)
            }
            R.id.ll_my_cancel -> {
                if (mAgreementDTO == null) {
                    showToast("暂未开通自动续费")
                } else {
                    val bundle = Bundle()
                    bundle.putSerializable(ManageOrderActivity.AGREEMENT_DATA, mAgreementDTO)
                    readyGo(ManageOrderActivity::class.java, bundle)
                }
            }
            R.id.ll_my_order -> {
                readyGo(OrderRecordActivity::class.java)
            }
            R.id.ll_my_phone -> {
                backPhoneDialog()
            }
            R.id.ll_my_set -> {
//                val bundle = Bundle()
//                bundle.putString(Constant.ROUTE_BOOK_ID, "80670")
//                readyGo(LongBookReadActivity::class.java, bundle)
                readyGo(SettingsActivity::class.java)
            }

        }
    }

    override fun getUserSuc(data: UserBean) {
        getData()
        loginStatus(data)
    }

    private fun loginStatus(consumer: UserBean) {
        // 是否有手机号
        if (!TextUtils.isEmpty(userManager.mobile)) {
            tv_my_account.text =
                "手机号:" + consumer.mobile.substring(0, 3) + "****" + consumer.mobile.substring(
                    consumer.mobile.length - 4
                )
            iv_my_bind.visibility = View.GONE
        } else {
            tv_my_account.text = "绑定手机号"
            iv_my_bind.visibility = View.VISIBLE
        }
        tv_my_id.text = "书友" + consumer.idcode

        // vip状态
        if (consumer.expireTime > 0) {
            iv_my_bind.setImageResource(R.mipmap.ic_my_v)
            tv_my_vip.text = "VIP会员"
            if (consumer.isForever) {
                tv_my_status.text = "有效期: 永久有效"
            } else {
                tv_my_status.text =
                    "到期时间 ${TimeRender.formatDate(consumer.expireTime, TimeRender.DEFAULT_FORMAT)}"
            }
            tv_my_btn.text = "已开通"
        } else {
            iv_my_bind.setImageResource(R.mipmap.ic_my_pt)
            tv_my_vip.text = "开通VIP会员"
            tv_my_status.text = "万本小说免费看"
            tv_my_btn.text = "立即开通"
        }
    }

    private fun initListener() {
        ll_my_login.setOnClickListener(this)
        tv_my_btn.setOnClickListener(this)
        tv_my_account.setOnClickListener(this)
        ll_my_history.setOnClickListener(this)
        ll_my_change.setOnClickListener(this)
        ll_my_cancel.setOnClickListener(this)
        ll_my_order.setOnClickListener(this)
        ll_my_phone.setOnClickListener(this)
        ll_my_set.setOnClickListener(this)
    }

    fun backPhoneDialog() {
        phoneDialog = PhoneDialog(requireContext(), object : DialogListener() {
            override fun onClick(v: View) {
                if (phoneDialog != null && phoneDialog!!.isShowing()) {
                    phoneDialog!!.dismiss()
                }
                when (v.id) {
                    R.id.tv_phone_dialog_phone, R.id.tv_phone_dialog_phone2 -> {
                        val intent = Intent(Intent.ACTION_DIAL)
                        intent.data = Uri.parse("tel:" + "4000969950")
                        requireContext().startActivity(intent)
                    }
                }
            }
        })
        phoneDialog!!.show()
    }

    private fun getData() {
        AppHttpManager.getInstance(baseApplication)
            .call(baseApplication.loanService.cashOutRecords,
                object : BaseSubscriber<BaseResponse<OrderRecordBean>>() {
                    override fun onError(e: Throwable) {
                    }

                    override fun onNext(t: BaseResponse<OrderRecordBean>) {
                        if (t.status == 200) {
                            mAgreementDTO = t.data.agreement
                        }
                    }
                })
    }
}