package com.duben.infinitefunjia.ui.widgets

import android.app.Dialog
import android.content.Context
import android.view.Gravity
import android.view.WindowManager
import android.widget.RadioButton
import android.widget.SeekBar
import com.duben.infinitefunjia.R
import com.duben.infinitefunjia.common.BookConstant
import kotlinx.android.synthetic.main.dialog_book_setting.*

class LongBookSettingDialog(context: Context) : Dialog(context, R.style.dialog) {

    private var mOnBookSettingCallback: OnLongBookSettingCallback? = null

    init {
        setContentView(R.layout.dialog_book_setting)
        // 设置window属性
        val lp = window!!.attributes
        lp.gravity = Gravity.BOTTOM
        lp.width = WindowManager.LayoutParams.MATCH_PARENT
        lp.windowAnimations = R.style.DialogAnimBottom
        // lp.dimAmount = 0f // 去背景遮盖
        lp.alpha = 1f//透明效果
        window!!.attributes = lp
        setCancelable(true)
        setCanceledOnTouchOutside(true)

        (seekbar as SeekBar).setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                tv_seek.text = "$progress"
                mOnBookSettingCallback?.onFontSize(progress)
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {}

            override fun onStopTrackingTouch(seekBar: SeekBar?) {}
        })

        rg_color.setOnCheckedChangeListener { group, checkedId ->
            for (i in 0 until group.childCount) {
                val rb = group.getChildAt(i) as RadioButton
                if (rb.id == checkedId) {
                    rb.scaleX = 1.2f
                    rb.scaleY = 1.2f
                } else {
                    rb.scaleX = 1f
                    rb.scaleY = 1f
                }
            }
            when (checkedId) {
                R.id.rb_color1 -> {
                    mOnBookSettingCallback?.onBgColor(BookConstant.BOOK_CONTENT_COLOR_1.color)
                }
                R.id.rb_color2 -> {
                    mOnBookSettingCallback?.onBgColor(BookConstant.BOOK_CONTENT_COLOR_2.color)
                }
                R.id.rb_color3 -> {
                    mOnBookSettingCallback?.onBgColor(BookConstant.BOOK_CONTENT_COLOR_3.color)
                }
                R.id.rb_color4 -> {
                    mOnBookSettingCallback?.onBgColor(BookConstant.BOOK_CONTENT_COLOR_4.color)
                }
                R.id.rb_color5 -> {
                    mOnBookSettingCallback?.onBgColor(BookConstant.BOOK_CONTENT_COLOR_5.color)
                }
                R.id.rb_color6 -> {
                    mOnBookSettingCallback?.onBgColor(BookConstant.BOOK_CONTENT_COLOR_6.color)
                }
                else -> {}
            }
        }
    }

    fun setSetting(color: Int? = null, size: Int? = null) {

        size?.let {
            tv_seek.text = "$it"
            seekbar.progress = it
        }

        when (color) {
            BookConstant.BOOK_CONTENT_COLOR_1.color -> {
                rb_color1.scaleX = 1.2f
                rb_color1.scaleY = 1.2f
                rb_color1.isChecked = true
            }
            BookConstant.BOOK_CONTENT_COLOR_2.color -> {
                rb_color2.scaleX = 1.2f
                rb_color2.scaleY = 1.2f
                rb_color2.isChecked = true
            }
            BookConstant.BOOK_CONTENT_COLOR_3.color -> {
                rb_color3.scaleX = 1.2f
                rb_color3.scaleY = 1.2f
                rb_color3.isChecked = true
            }
            BookConstant.BOOK_CONTENT_COLOR_4.color -> {
                rb_color4.scaleX = 1.2f
                rb_color4.scaleY = 1.2f
                rb_color4.isChecked = true
            }
            BookConstant.BOOK_CONTENT_COLOR_5.color -> {
                rb_color5.scaleX = 1.2f
                rb_color5.scaleY = 1.2f
                rb_color5.isChecked = true
            }
            BookConstant.BOOK_CONTENT_COLOR_6.color -> {
                rb_color6.scaleX = 1.2f
                rb_color6.scaleY = 1.2f
                rb_color6.isChecked = true
            }
        }
    }

    fun setOnBookSettingCallback(onLongBookSettingCallback: OnLongBookSettingCallback) {
        this.mOnBookSettingCallback = onLongBookSettingCallback
    }

    interface OnLongBookSettingCallback {
        fun onFontSize(size: Int)
        fun onBgColor(color: Int)
    }


}