package com.duben.infinitefunjia.ui.widgets;

import android.os.CountDownTimer;

/**
 * 简单 时，分，秒，毫秒倒计时
 * <p>
 * 精确到毫秒
 */

public class MilliDownTimer extends CountDownTimer {

    // 默认倒计时间隔
    private static final long DEFAULT_INTERVAL = 100L;

    /**
     * 秒，分，时对应的毫秒数
     */
    private int sec = 1000, min = sec * 60, hr = min * 60;

    /**
     * 结束监听
     */
    private OnFinishListener onFinishListener;

    private OnTickListener onTickListener;

    /**
     * @param millisInFuture    倒计时总时间 单位：毫秒
     *                          The number of millis in the future from the call
     *                          to {@link #start()} until the countdown is done and {@link #onFinish()}
     *                          is called.
     * @param countDownInterval 倒计时间隔 单位：毫秒
     *                          The interval along the way to receive
     *                          {@link #onTick(long)} callbacks.
     */
    public MilliDownTimer(long millisInFuture, long countDownInterval) {
        super(millisInFuture, countDownInterval);
    }

    /**
     * @param millisInFuture 倒计时总时间 单位：毫秒
     *                       The number of millis in the future from the call
     *                       to {@link #start()} until the countdown is done and {@link #onFinish()}
     *                       is called.
     */
    public MilliDownTimer(long millisInFuture) {
        super(millisInFuture, DEFAULT_INTERVAL);
    }

    /**
     * 每一次间隔时间到后调用
     *
     * @param millisUntilFinished 剩余总时间 单位：毫秒
     */
    @Override
    public void onTick(long millisUntilFinished) {
        // 剩余的小时，分钟，秒，毫秒
        long lHr = millisUntilFinished / hr;
        long lMin = (millisUntilFinished - lHr * hr) / min;
        long lSec = (millisUntilFinished - lHr * hr - lMin * min) / sec;
        long lMs = millisUntilFinished - lHr * hr - lMin * min - lSec * sec;
        String strLHr = getTime(lHr);
        String strLMin = getTime(lMin);
        String strLSec = getTime(lSec);
        String strLMs = getMs(lMs);
        // 依次拼接时间 时:分:秒:毫秒

        if (onTickListener != null) {
            onTickListener.onTick(strLMin, strLSec, strLMs);
        }
    }

    /**
     * 根据毫秒换算成相应单位后是否大于10来返回相应时间
     */
    private String getTime(long time) {
        return time > 10 ? String.valueOf(time) : "0" + time;
    }

    /**
     * 获取毫秒
     */
    private String getMs(long time) {
        String strMs = String.valueOf(time);
        return time > 100 ? strMs.substring(0, strMs.length() - 1) : "00";
    }

    /**
     * 结束监听，可以在倒计时结束时做一些事
     */
    public interface OnFinishListener {
        void onFinish();
    }

    /**
     * 倒计时监听
     */
    public interface OnTickListener {
        void onTick(String min, String sec, String mil);
    }

    /**
     * 设置/**
     */
    public MilliDownTimer setOnTickListener(OnTickListener onTickListener) {
        this.onTickListener = onTickListener;
        return this;
    }

    /**
     * 设置结束监听
     *
     * @param onFinishListener 结束监听对象
     */
    public MilliDownTimer setOnFinishListener(OnFinishListener onFinishListener) {
        this.onFinishListener = onFinishListener;
        return this;
    }

    /**
     * 倒计时结束时调用
     */
    @Override
    public void onFinish() {
        if (onFinishListener != null)
            onFinishListener.onFinish();
    }
}
