package com.duben.infinitefunjia.utils;

public class GpsUtil {

    private static final double R = 6371;// 地球半径千米

    //计算纬度半径差
    private static double dlat(double dis) {

        double dlat = dis / R;
        return dlat * 180 / Math.PI;
    }

    //计算经度半径差
    private static double dlng(double latitude, double dis) {
        double dlng = 2 * Math.asin(Math.sin(dis / (2 * R)) / Math.cos(latitude * Math.PI / 180));
        return dlng * 180 / Math.PI;// 角度转为弧度
    }


    /**
     * 计算该GPS半径在radius(默认50米)的区域的范围（矩形范围）
     * @param lat 纬度
     * @param lng 经度
     * @param radius 半径
     * @return
     */
    private static GpsRange getGpsRange(double lat, double lng, double radius) {
        if (radius <= 0)
            radius = 0.05;
        double longitude = lng;// 经度
        double dlng = dlng(lat, radius);//经度半径差
        double latitude = lat;// 纬度
        double dlat = dlat(radius);//纬度半径差
        GpsRange gpsRange = new GpsRange();
        gpsRange.lat_low = latitude - dlat;//左纬度
        gpsRange.lat_high = latitude + dlat;//右纬度
        gpsRange.lng_low = longitude - dlng;//上经度
        gpsRange.lng_high = longitude + dlng;//下经度
        return gpsRange;
    }

    static class GpsRange{
        public double lat_low,lat_high;//左纬度,右纬度
        public double lng_low,lng_high;//上经度,下经度
    }

    /**
     * 验证2个Gps坐标是否在一定范围内(半径)交叉
     * @param longitude1 经度1
     * @param latitude1  纬度1
     * @param longitude2 经度2
     * @param latitude2  纬度2
     * @param radius
     * @return
     */
    private static boolean checkCross(double longitude1, double latitude1,double longitude2, double latitude2, double radius){
        if (longitude1 <= 0 || latitude1 <=0 || longitude2<=0 || latitude2 <=0)
            throw new RuntimeException("经纬度错误");
        GpsRange gpsRange = getGpsRange(latitude1, longitude1, radius);
        System.out.println("经纬度:（"+longitude2+"|"+latitude2+"）"+gpsRange.lng_low +"--"+ gpsRange.lng_high  +"--"+gpsRange.lat_low+ "--"+gpsRange.lat_high);

        if (longitude2 >= gpsRange.lng_low && longitude2 <= gpsRange.lng_high && latitude2 >= gpsRange.lat_low && latitude2 <= gpsRange.lat_high)
        return true;

        return false;
    }

    /**
     *
     * @param longitude1
     * @param latitude1
     * @param longitude2
     * @param latitude2
     * @param radius
     * @return true-范围内
     */
    public static boolean checkCrossM(double longitude1, double latitude1,double longitude2, double latitude2, double radius){
        return checkCross(longitude1,latitude1,longitude2,latitude2,radius/1000);
    }


}