package com.duben.infinitefunjia.utils

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import com.permissionx.guolindev.PermissionX


object PermissionXUtils {

    /**
     * 位置权限
     */
    fun requestLocationPermission(
        ctx: Fragment,
        callback: Callback
    ) {
        val permissions: List<String> = if (Build.VERSION.SDK_INT >= 29) {
            listOf(
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_BACKGROUND_LOCATION
            )
        } else {
            listOf(
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION
            )
        }

        PermissionX.init(ctx)
            .permissions(permissions)
            .request { allGranted, _, _ ->
                if (allGranted) {
                    callback.success()
                } else {
                    callback.fail()
                }
            }
    }

    fun checkPermission2(
        ctx: Fragment,
        permissions: List<String>,
        callback: Callback
    ) {
        PermissionX.init(ctx)
            .permissions(permissions)
            .request { allGranted, _, _ ->
                if (allGranted) {
                    callback.success()
                } else {
                    if (checkLocationPermission(ctx.requireContext(), permissions))
                        callback.success()
                    else
                        callback.fail()
                }
            }
    }

    fun checkLocationPermission(ctx: Context, permissions: List<String>): Boolean {
        //是否是定位权限判断
        var isLocation = false
        var failCount = 0
        for (perm in permissions) {
            if (Manifest.permission.ACCESS_BACKGROUND_LOCATION == perm) {
                isLocation = true
            }
            if (ContextCompat.checkSelfPermission(
                    ctx,
                    perm
                ) !== PackageManager.PERMISSION_GRANTED
            ) {
                ++failCount
            }
        }
        return if (isLocation) {
            //定位权限的判断
            //当点击仅在使用中允许 或 全部允许 失败数量为1  ，  0 但是这两种是都可以成功定位的
            failCount <= 1
        } else {
            //没有安卓10的定位权限
            failCount == 0
        }
    }


    interface Callback {
        fun success()
        fun fail()
    }


}